/*  DT_drawtext.c
 *  Written By: Garrett Banuk <mongoose@wpi.edu>
 *  This is free, just be sure to give me credit when using it
 *  in any of your programs.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include "DT_drawtext.h"

#define N_CHARS 94

static BitFont *BitFonts = NULL;  /* Linked list of fonts */

/* Loads the font into a new struct 
 * returns -1 as an error else it returns the number
 * of the font for the user to use
 */
int DT_LoadFont(const char *BitmapName, int flags)
{
  int    FontNumber = 0;
  BitFont  **CurrentFont = &BitFonts;
  SDL_Surface  *Temp;

  
  while(*CurrentFont)
  {
    CurrentFont = &((*CurrentFont)->NextFont);
    FontNumber++;
  }
  
  /* load the font bitmap */
  if(NULL ==  (Temp = SDL_LoadBMP(BitmapName)))
  {
    fprintf(stderr, "ERROR: Cannot load font file %s\n", BitmapName );
    return -1;
  }

  /* Add a font to the list */
  //*CurrentFont = (BitFont*)malloc(sizeof(BitFont));
  //*CurrentFont = &SystemFont;
  *CurrentFont = new BitFont;
         
  (*CurrentFont)->FontSurface = SDL_DisplayFormat(Temp);
  SDL_FreeSurface(Temp);

  (*CurrentFont)->CharWidth = (*CurrentFont)->FontSurface->w/N_CHARS;
  (*CurrentFont)->CharHeight = (*CurrentFont)->FontSurface->h;
  (*CurrentFont)->FontNumber = FontNumber;
  (*CurrentFont)->NextFont = NULL;


  /* Set font as transparent if the flag is set */
  if(flags & TRANS_FONT)
  {
    /* This line was left in in case of problems getting the pixel format */
    /* SDL_SetColorKey((*CurrentFont)->FontSurface, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0xFF00FF); */
    SDL_SetColorKey((*CurrentFont)->FontSurface, SDL_SRCCOLORKEY|SDL_RLEACCEL,
             (*CurrentFont)->FontSurface->format->Rmask|(*CurrentFont)->FontSurface->format->Bmask);
  }
    
  //fprintf(stderr, "*** Font \"%s\" loaded. Width:%d, Height:%d\n\n", BitmapName, (*CurrentFont)->CharWidth, (*CurrentFont)->CharHeight );
  return FontNumber;
}


/* Takes the font type, coords, and text to draw to the surface*/
void DT_DrawText(const char *string, SDL_Surface *surface, int FontType, int x, int y, int tp)
{
  int      loop;
  int      characters;
  SDL_Rect  SourceRect, DestRect;
  BitFont    *CurrentFont;


  CurrentFont = DT_FontPointer(FontType);

  /* see how many characters can fit on the screen */
  if(x>surface->w || y>surface->h) return;

  if(strlen(string) < (surface->w-x)/CurrentFont->CharWidth)
    characters = strlen(string);
  else
    characters = (surface->w-x)/CurrentFont->CharWidth;

  DestRect.x = x;
  DestRect.y = y;
  DestRect.w = CurrentFont->CharWidth;
  DestRect.h = CurrentFont->CharHeight;

  SourceRect.y = 0;
  SourceRect.w = CurrentFont->CharWidth;
  SourceRect.h = CurrentFont->CharHeight;

  if (!tp)
  {
    SDL_Rect dr;
    
    dr.x = x;
    dr.y = y;
    
    dr.h = DestRect.h;
    dr.w = DestRect.w * characters;
    
    SDL_FillRect(surface, &dr, 0);
  }

  /* Now draw it */
  int x1 = 0;
  for(loop=0; loop<characters; loop++)
  {
    x1 = (string[loop] - 33) * CurrentFont->CharWidth;
    if (x1 >= CurrentFont->FontSurface->w || x1 < 0) 
      x1 = CurrentFont->FontSurface->w - CurrentFont->CharWidth;  

    SourceRect.x = x1;
    if (string[loop] >= 33)
      SDL_BlitSurface(CurrentFont->FontSurface, &SourceRect, surface, &DestRect);
    
    DestRect.x += CurrentFont->CharWidth;
  }
}

/* Returns the height of the font numbers character
 * returns 0 if the fontnumber was invalid */
int DT_FontHeight( int FontNumber )
{
  BitFont    *CurrentFont;

  CurrentFont = DT_FontPointer(FontNumber);
  if(CurrentFont)
    return CurrentFont->CharHeight;
  else
    return 0;
}

/* Returns the width of the font numbers charcter */
int DT_FontWidth( int FontNumber )
{
  BitFont    *CurrentFont;

  CurrentFont = DT_FontPointer(FontNumber);
  if(CurrentFont)
    return CurrentFont->CharWidth;
  else
    return 0;
}

/* Returns a pointer to the font struct of the number
 * returns NULL if theres an error
 */
BitFont* DT_FontPointer(int FontNumber)
{
  BitFont *CurrentFont = BitFonts;
  BitFont *temp;

  while(CurrentFont)
    if(CurrentFont->FontNumber == FontNumber)
      return CurrentFont;
    else
    {
      temp = CurrentFont;
      CurrentFont = CurrentFont->NextFont;
    }
  
  return NULL;

}

/* removes all the fonts currently loaded */
void DT_DestroyDrawText()
{
  BitFont *CurrentFont = BitFonts;
  BitFont *temp;

  while(CurrentFont)
  {
    temp = CurrentFont;
    CurrentFont = CurrentFont->NextFont;
    
    SDL_FreeSurface(temp->FontSurface);
    delete(temp);
  }

  BitFonts = NULL;
}
