/*
 *  decoder.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */ 
#ifndef _MPEG2DEC_H
#define _MPEG2DEC_H 1

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <inttypes.h>
#include <unistd.h>
#include <malloc.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include "mpeg2.h"
#include "avcodec.h"
#include "mpegaudio.h"

#define V_CHUNK_BUF_LEN (256*1024)

#define ALLOC_YUV 2
//extern void *mpeg2_malloc(int size, int reason);
#define mpeg2_malloc(size, reason) memalign(16, size)

#ifdef __cplusplus
}
#endif

#include "frame_srv.h"

class T_MPEG_DEC;


//============================================================================
//==                             A U D I O                                  ==
//============================================================================

typedef struct 
{
  char          tag1[4];
  unsigned char seg_len1[4];
  char          fourcc[4];
  
  char          tag2[4];
  unsigned char seg_len2[4];
  unsigned char stereo[2];
  unsigned char channels[2];
  unsigned char sample_rate[4];
  unsigned char bit_rate[4];
  unsigned char Bps[2];
  unsigned char bps[2];
  
  char          tag3[4];
  unsigned char seg_len3[4];
  
} T_WAV_HEADER;

//=======================
//== class T_AUDIO_DEC ==
//=======================
class T_AUDIO_DEC
{
  protected:
    FILE   *af;
    int    act_frame;
    int    sample_rate;
    int    bit_rate;
    int    track_no;
    double video_rate;

    double samples_pvf;  
    int    samples_paf;
    int    size_paf;
      
    T_FRAME_SRV *act_frameSrv;
    T_MPEG_DEC  *v_decoder;

    int     bytes_used;
    int     bytes_unused;
    int     refilled;
  
    CodecID ffmpeg_id;
    
  public:
    int  needDemux;
    
    static int avs_dp;
        
  public:
    static T_AUDIO_DEC *createAudioDec(T_FRAME_SRV *frame_srv, int track);

    T_AUDIO_DEC(int track);
    ~T_AUDIO_DEC();

    virtual int  open(T_FRAME_SRV *frame_srv, T_MPEG_DEC *video_dec);
    virtual void close(void);
    virtual int  readSamples(uint32_t *sample_buf, double *samples_cnt);
    virtual int  readChunkData(uint8_t *chunk_buf, double *samples_cnt);
    
    CodecID     getCodecId(void);
    
    int         getSampleRate(void);
    double      getSamplesPVF(void);
    T_FRAME_SRV *getFrameSrv(void);
    int         getEncodedFs(void);
    
    void	resetBuffer(int p);
};


//============================
//== class T_AUDIO_DEC_NONE ==
//============================
class T_AUDIO_DEC_NONE : public T_AUDIO_DEC
{
  public:
    T_AUDIO_DEC_NONE(void) : T_AUDIO_DEC(0) {};

    virtual int  open(T_FRAME_SRV *frame_srv, T_MPEG_DEC *video_dec);
};

//===========================
//== class T_AUDIO_DEC_WAV ==
//===========================
class T_AUDIO_DEC_WAV : public T_AUDIO_DEC
{
  public:
    T_AUDIO_DEC_WAV(void) : T_AUDIO_DEC(0) {};

    virtual int  open(T_FRAME_SRV *frame_srv, T_MPEG_DEC *video_dec);
    virtual void close(void);
    virtual int  readSamples(uint32_t *sample_buf, double *samples_cnt);
};

//==============================
//== class T_AUDIO_DEC_FFMPEG ==
//==============================
class T_AUDIO_DEC_FFMPEG : public T_AUDIO_DEC
{
  private:
    AVCodec        *ffmpeg_codec;
    AVCodecContext *ffmpeg_ctx;
    
    int     isOpen;      

  private:    
    int     decode(int8_t *sample_buf, double *samples_cnt);
    
  public:
    T_AUDIO_DEC_FFMPEG(CodecID id, int track);

    virtual int  open(T_FRAME_SRV *frame_srv, T_MPEG_DEC *video_dec);
    virtual void close(void);
    virtual int  readSamples(uint32_t *sample_buf, double *samples_cnt);

    virtual int  readChunkData(uint8_t *chunk_buf, double *samples_cnt);
};

//============================================================================
//==                             V I D E O                                  ==
//============================================================================

//========================
//==   class T_MPEG_DEC ==
//========================
class T_MPEG_DEC
{
  private:
    uint8_t       chunk_buf[V_CHUNK_BUF_LEN];
    mpeg2dec_t    *decoder;
    uint32_t	  accel;

    static uint8_t *vo_pages[3];    
    uint8_t        vo_visible;
    int            vo_size_luma;
    int            vo_size_chroma;

    int           N_BUF;
    int           nFrames;
    int           p_n_buf;
    int		  act_frame;
    int           p_sav;
    T_FRAME_SRV   *act_frameSrv;
    char          rebuild_idx;

    T_AUDIO_DEC   *audio_dec;    
    int           needADemux;
    
    char          closed_gop;
    
    static int    ff_init_done;
        
  private:
    int         init(void);
    void        cleanup(void);
    int         decode(uint8_t * current, uint8_t * end);
    
  public:
    T_MPEG_DEC();
    ~T_MPEG_DEC();
    
    void        forceIndex(void);
    
    T_FRAME_SRV *open(char *fname, int *p);
    int         open(T_FRAME_SRV *fs, int *p);
    
    void        close(T_FRAME_SRV *frame_srv);
    
    int         decodeFrame(T_FRAME_SRV *frame_srv, int p, int *step, 
                            uint8_t *py, uint8_t *pu, uint8_t *pv);
    int         readFrame(T_FRAME_SRV *frame_srv, int p, 
                          uint8_t *pchunk, int *len, int gop_bound);
                            
    int         getActFrame(void);
    
    T_AUDIO_DEC *getAudioDecoder(void);
    int         openAudio(T_FRAME_SRV* frame_srv, int track);
    void        closeAudio(void);
   
    void        setAudioSync(int dp);
    int         getBitRate(void);

  public:
    uint8_t	  mSeqHdr[128];
    int           mSeqHdrLen;
};

#endif
