/*
 *  edit.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include "lve_gui.h"
#include "edit.h"

void T_MB_EDIT::Save(FILE *lst_file)
{
  T_MEDIA_BUF media_buf;

  media_buf.media_id    = mdf->getId();
  media_buf.vstream     = vstream;
  media_buf.astream     = ctx->a_stream;  // astream;
  media_buf.start       = start;
  media_buf.end         = end;
  media_buf.dist        = dist;
  media_buf.start_real  = start_real;
  media_buf.end_real    = end_real;
  media_buf.avs_dp      = avs_dp;
  strcpy(media_buf.fname, mdf->getName());
  memcpy(media_buf.bmp_thumb, bmp_thumb, BMP_TH_LEN);  
  
  fwrite(&media_buf, sizeof(T_MEDIA_BUF), 1, lst_file);
}

int T_MB_EDIT::Load(FILE *lst_file)
{
  T_MEDIA_BUF media_buf;
  int         n = 0;
  TMediaFile  *_mdf;
  
  //-- read scene record into buffer -- 
  //-----------------------------------
  n = fread(&media_buf, sizeof(T_MEDIA_BUF), 1, lst_file);
  
  if (n != 0)
  {
    //-- create reference to media file and setup scene --
    //----------------------------------------------------
    _mdf = TMediaFile::createMDF(media_buf.fname);
    if (!_mdf) return -1;
    
    if (!Setup(_mdf, media_buf.astream, media_buf.vstream) )
               
    {
      TMediaFile::releaseMDF(_mdf);
      mdf = NULL;
      return -1;
    }
                
    start         = media_buf.start;
    end           = media_buf.end;
    dist          = media_buf.dist;
    start_real    = media_buf.start_real;
    end_real      = media_buf.end_real;
    avs_dp        = media_buf.avs_dp;
    
    memcpy(bmp_thumb, media_buf.bmp_thumb,  BMP_TH_LEN);  
  }

  return (n==0);
}

int T_MB_EDIT::TrimLeft(void)
{
  int ncut = ctx->act_real - start_real;
  
  //-- nothing to do --
  //if (ncut <= 0) return 0;
  if (!ncut) return 0;
  
  start_real  = ctx->act_real;
  dist       -= ncut;
  end        -= ncut;

  return ncut;
}

int T_MB_EDIT::TrimRight()
{
  int ncut = end_real - ctx->act_real + 1; 
  
  //-- nothing to do --
  if (!ncut) return 0;

  end_real  = ctx->act_real - 1; 
  dist     -= ncut;
  end      -= ncut;

  return ncut;
}

T_MB_SCENE *T_MB_EDIT::Dup(void)
{
  T_MB_EDIT *mb_dup;
  mb_dup = new T_MB_EDIT(ctx, 0);
  
  if (mb_dup)     
  {
    mb_dup->mdf         = mdf->dup();
    mb_dup->astream     = astream;
    mb_dup->vstream     = vstream;

    mb_dup->dist        = dist;
    mb_dup->start_real  = start_real;
    mb_dup->end_real    = end_real;
    mb_dup->start       = start;
    mb_dup->end         = end;
    mb_dup->avs_dp      = avs_dp;
      
    memcpy(mb_dup->bmp_thumb, bmp_thumb, BMP_TH_LEN);
    
    mb_dup->has_mark    = 0;

    return mb_dup;
  }  

  fprintf(stderr, "ERROR: T_MB_MEDIA::Dup() - out of memory on dup !\n"); 
  return NULL;
}


//=============================================================

void T_LST_EDIT::RemoveMember(T_MEMBER *mb_cut, int del)
{
  T_MB_EDIT *mb;
  T_MB_EDIT *mb_rem = (T_MB_EDIT *)mb_cut; 
  
  if (mb_rem)
  {
    dist_sum -= mb_rem->dist;
    ctx->dist_sum = dist_sum;
    
    mb     = (T_MB_EDIT *)mb_rem->GetNext();
    while (mb)
    {
      mb->start -= mb_rem->dist;
      mb->end   -= mb_rem->dist;
      
      mb = (T_MB_EDIT *)mb->GetNext();
    }
    T_LIST::RemoveMember(mb_rem, del);
  }
}

T_MEMBER *T_LST_EDIT::InsertMember(T_MEMBER *mb_new, T_MEMBER *mb)
{
  T_MB_SCENE *mb1 = (T_MB_SCENE *)mb;
  T_MB_SCENE *mb_new1;

  if ( (mb_new1 = (T_MB_SCENE *)T_LIST::InsertMember(mb_new, mb)) != NULL )
  {
    //-- for new member take start-frame from  --
    //-- next member and calculate its end pos --
    if ( mb1 != NULL )
    {
      mb_new1->start = mb1->start;
      mb_new1->end   = mb1->start + mb_new1->dist - 1;
      
      //-- adjust start/end position from rest of members --
      while (mb1)
      {
        mb1->start += mb_new1->dist;
        mb1->end   += mb_new1->dist;
        mb1 = (T_MB_EDIT *)mb1->GetNext();
      }
    }
    dist_sum += mb_new1->dist;
    ctx->dist_sum = dist_sum;
  } 
  return mb_new1; 
}

T_MEMBER *T_LST_EDIT::AppendMember(T_MEMBER *_mb_new)
{
  T_MB_EDIT *mb_new = (T_MB_EDIT *)_mb_new; 
  T_MB_EDIT *mb     = NULL;
  
  if ( (mb = (T_MB_EDIT *)T_LIST::AppendMember(mb_new)) != NULL)
  {
    //-- adjust number of new member -- 
    mb_new->mb_num = mb_cnt-1;
    
    //-- adjust frames counts of list and new member --
    mb_new->start = dist_sum;
    dist_sum += mb_new->dist;
    mb_new->end = dist_sum - 1;
  
    ctx->dist_sum = dist_sum;
  }
  return _mb_new;
}

//========================

void T_LST_EDIT::Reset()
{
  T_LIST::Reset();
 
  dist_sum = 0;
  mb_cnt   = 0;
   
  // never reset the media-id !!!
}

int T_LST_EDIT::SaveHeader(FILE *lst_file)
{
  int64_t out_size = 0;
  
  if (dist_sum > 0)
  {   
    strcpy(lst_header.magic, EDIT_BUF_MAGIC);
    lst_header.dist_sum = dist_sum;
    lst_header.mb_cnt   = mb_cnt;
    
    //-- estimate total size for dumped (output) file --
    lst_header.output_size_mb = GetOutputSize() / (1024*1024);
    
    fwrite(&lst_header, sizeof(T_MEDIA_HDR), 1, lst_file);
    return 1;
  }
  return 0;
}

int T_LST_EDIT::LoadHeader(FILE *lst_file)
{
  int rc = 0;

  rc = fread(&lst_header, sizeof(T_MEDIA_HDR), 1, lst_file);
  if (rc) rc = ( 0 == strcmp(lst_header.magic, EDIT_BUF_MAGIC) );
  
  if (!rc)
  {
    fprintf(stderr,"WARNING: this is no edit-list file\n");
    media_id = 0;
    dist_sum = 0;
    mb_cnt   = 0;  
   
    return 0;
  }
  else
  {
    dist_sum = lst_header.dist_sum;
    
    return 1;  
  }
}

void T_LST_EDIT::CommitHeader(void)
{
  dist_sum = lst_header.dist_sum;
}

T_MEMBER* T_LST_EDIT::CreateMember(int n)
{
  return (T_MEMBER *)new T_MB_EDIT(ctx, n);
} 

void T_LST_EDIT::TrimMember(T_MB_EDIT *mb, int side, int draw_thumb)
{
  T_MB_EDIT *mb_trim;
  int       ncut;
  
  if (!mb) return;
  
  if (side == 0)
    ncut = mb->TrimLeft();
  else
    ncut = mb->TrimRight();    
 
  if (ncut == 0) return; 

  //-- adjust total number of frames --
  dist_sum -= ncut;
  
  ctx->dist_sum = dist_sum;
  ctx->start    = mb->GetStartFrame();
  ctx->end      = mb->GetEndFrame();
   
  //-- adjust thumb-nail only for trim left --
  if (!side && draw_thumb) mb->GenThumb(); 
 
  //-- adjust rest of list --
  mb_trim = (T_MB_EDIT *)mb->GetNext();
  while (mb_trim)
  {
    mb_trim->start -= ncut;
    mb_trim->end   -= ncut;
    mb_trim = (T_MB_EDIT *)mb_trim->GetNext();
  }
  
  mb_save_needed = 1;  
}

// eof
