/*
 *  ffmpeg_aenc.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "avcodec.h"

#define _FFMPEG_AENC_ 1
#include "ffmpeg_aenc.h"

void ffmpeg_aencode_close(EncoderCtx *enc)
{
  if (enc->a_ctx)
  {
    avcodec_close(enc->a_ctx);
    free(enc->a_ctx);
    enc->a_ctx = NULL;
  }   
}
 
EncoderCtx *ffmpeg_aencode_open(int bit_rate, int sample_rate)
{
  static EncoderCtx encoder;
  AVCodecContext    *a_ctx;
  
  register_avcodec(&mp2_encoder);
  
  if ( (encoder.a_ctx = avcodec_alloc_context()) != NULL)
  {  
    a_ctx = encoder.a_ctx;  
    
    //-- setup context --
    //a_ctx->codec_type  = CODEC_TYPE_AUDIO;
    //a_ctx->codec_id    = of->audio_codec;
    a_ctx->bit_rate    = bit_rate;
    a_ctx->channels    = 2;
    a_ctx->sample_rate = sample_rate;
      
    if (avcodec_open(a_ctx, &mp2_encoder) < 0)
    {
      fprintf(stderr, "ERROR: opening of audio encoder failed!\n");
      free(encoder.a_ctx);
      encoder.a_ctx = NULL;
      return NULL;
    }
    
    encoder.bytes_avail      = 0;
    encoder.sample_bytes_paf = a_ctx->frame_size * 4; // should be 1152 * 4 for MPA
    
    return &encoder;
  }
  
  return NULL;      
}

//== audio encoding:                                            ==
//==  i/o buffer will hold samples (decoded data) on entry. its ==
//==  content will be appended first to sample_buf. then it is  ==
//==  free to store encoded output data. bytes_pf is the amount ==
//==  of sample bytes needed to encode 1 audio frame (pack).    ==
//================================================================   
int ffmpeg_aencode(EncoderCtx *enc, uint8_t *iobuf, int iobuf_len, int max_iobuf_len)  
{
  uint8_t     *in_buf     = enc->sample_buf;
  int         bytes_free  = max_iobuf_len;
  int         bytes_coded = 0;
  int         cnt, size;
  
  //-- append new chunk data to this of last run --
  //-----------------------------------------------
  memcpy(in_buf + enc->bytes_avail, iobuf, iobuf_len);
  enc->bytes_avail += iobuf_len;
 
  //-- encode only full audio frames -- 
  //-----------------------------------
  cnt = 0;
  while (enc->bytes_avail >= enc->sample_bytes_paf)
  {
    size = avcodec_encode_audio(enc->a_ctx, iobuf, bytes_free, (short *)in_buf);
    if (!cnt) enc->encoded_fs = size;
    cnt++;
       
    iobuf            += size;
    bytes_free       -= size;
    bytes_coded      += size;
    
    in_buf           += enc->sample_bytes_paf;
    enc->bytes_avail -= enc->sample_bytes_paf;
  }
   
  //-- left samples will be hold in sample_buf until next run --
  //------------------------------------------------------------
  if (enc->bytes_avail) memmove(enc->sample_buf, in_buf, enc->bytes_avail);

  return bytes_coded; 
}   

// eof
