/*
 *  lve_config.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#define _LVE_CONFIG_ 1
#include "lve_config.h"

#define LC_SUB_DIR_WORK  "/.lve/"
#define LC_FN_AUTO_MDLST "lvemedia.lst"
#define LC_FN_AUTO_EDLST "lveedit.lst"
#define LC_MAX_SIZE      (sizeof(LC_SUB_DIR_WORK)+sizeof(LC_FN_AUTO_MDLST)+sizeof(LC_FN_AUTO_EDLST)+8) 

static int  lc_init_done = 0;
static char *lc_work_dir = NULL;
static int  lc_len       = 0;

void lc_init(void)
{
  if (!lc_init_done)
  {
    struct stat stat_buf;
    char   *home = getenv("HOME");
    
    if (home)
      lc_work_dir = (char *)malloc( strlen(home)+LC_MAX_SIZE );  
  
    if (lc_work_dir)
    {  
      strcpy(lc_work_dir, home);
      strcat(lc_work_dir, LC_SUB_DIR_WORK);

      if ( stat(lc_work_dir, &stat_buf) == -1 ) 
      {
        if ( mkdir(lc_work_dir, 0770) != -1 )
        {
          lc_len = strlen(lc_work_dir);
        }  
      }
      else
        lc_len = strlen(lc_work_dir);
    }
    lc_init_done = 1;
  }
}

char *lc_getWorkDir(void)
{
  if (!lc_init_done) lc_init();
  return (lc_work_dir);
}

char *lc_getAutoMDList(void)
{
  if (!lc_init_done) lc_init();

  if (lc_len)
  {
    lc_work_dir[lc_len] = '\0';
    strcat(lc_work_dir, LC_FN_AUTO_MDLST);
    return (lc_work_dir);
  }
  else
    return "tmp/" LC_FN_AUTO_MDLST;
}

char *lc_getAutoEDList(void)
{
  if (!lc_init_done) lc_init();

  if (lc_len)
  {
    lc_work_dir[lc_len] = '\0';
    strcat(lc_work_dir, LC_FN_AUTO_EDLST);
    return (lc_work_dir);
  }
  else
    return "tmp/" LC_FN_AUTO_EDLST;
}

//eof
