/*
 *  lve_ev.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <SDL/SDL.h>
#include "lve.h"
#include "region.h"
#include "lve_gui.h"
#include "lve_ev.h"

//== clear event queue ==
//=======================
void lve_ev_discard(void)
{
  SDL_Event event;
  while (SDL_PollEvent(&event));
}

//== main event handler entrance ==
//=================================
int lve_ev(T_LVE_CTX *ctx)
{
    SDL_Event event;
    SDLMod    mod;
    int       key, quit;  

    quit = 0;
    if (!SDL_PollEvent(&event)) return quit;
    
    if (event.type == SDL_ACTIVEEVENT)
    {
      if ( (event.active.state & (SDL_APPACTIVE | SDL_APPINPUTFOCUS)) != 0 )
      {
        if (event.active.gain) 
        {
          ctx->refresh_bg = 2;
          display_refresh(ctx);
        }
      }
    }
    
    //-- lock event handling --
    key = T_REGION::rg_event_select(&event);

    if (event.type != SDL_KEYDOWN) 
    {
     int t;
     
     switch(key)
     {
      case SDLK_i:
        {
          T_LST_SCENE *lst; 
          if ( (lst = (T_LST_SCENE *)T_REGION::GetListGlobal()) != NULL )
          {
            T_MB_SCENE *mb;
            if ( (mb = (T_MB_SCENE *)lst->GetFocusMember()) != NULL )
              mb->ShowInfo(lst);
          }
        }
        break;
        
      //-- switch to another audio track --
      case SDLK_0:
      case SDLK_1:
      case SDLK_2:
      case SDLK_3:
      case SDLK_4:
        ctx->action_sav = ctx->action;
        t               = key - SDLK_0;
        if (t < ctx->file->getTotalAudioStreams())
        {
          if (t != ctx->a_stream)
          {
            fprintf(stderr, "INFO: lve_ev - switching to audio track (%d)\n", t); 
            ctx->a_stream = t;
            ctx->action   = A_ATRACK;
          }
          else
            fprintf(stderr, "INFO: lve_ev - audio track (%d) already activ\n", t); 
        }
        else
          fprintf(stderr, "INFO: lve_ev - audio track (%d) not available\n", t);  
        break;
      
      //-- quit program --
      case SDLK_q:
        REGION(RG_ID_PLAY).SwitchOff(0);
        quit = 1;
        break;
        
      //-- decrease speed --    
      case SDLK_r:
        mod = SDL_GetModState();
        if ( (mod & KMOD_CTRL) && (mod & KMOD_ALT) )
          lve_reset_lists(ctx);
        break;
 
      //-- cut media-scene --
      case SDLK_TAB:
        ((T_RG_MEDIA &)REGION(RG_ID_MEDIA)).SplitVideo(); 
        break;

      //-- join media-scenes --
      case SDLK_BACKSPACE:
        ((T_RG_MEDIA &)REGION(RG_ID_MEDIA)).JoinVideo(); 
        break;
        
      case SDLK_SPACE:
        T_MB_MEDIA *mbm;
        mbm  = (T_MB_MEDIA *)REGION(RG_ID_MEDIA).GetList()->GetFocusMember();
        if (mbm) 
        {
          mbm->GenThumb();
          REGION(RG_ID_MEDIA).DisplayUpdate();
        }
        break;  

      default:
        ;
     } 
    }
    return quit;
}

// eof
