/*
 *  lve_gui.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include "lve_gui.h"
#include "DT_drawtext.h"

static int step_multi = 0;
static int trim_mode  = 1;

//----------------------------------------------------------------------------

T_APP::T_APP(T_LVE_CTX *ctx, char *lib_path)
{
  //-- common setup for all regions --
  T_REGION::rg_setup(ctx, lib_path);   

  //-- create instances now ... -- 
  CreateRegions();
  
  //-- ... and sort it by ID for speed optimized access --
  T_REGION::rg_sort();
   
  return;
}

void T_APP::CreateRegions(void)
{
  //-- single regions --
  new T_RG_OV(RG_ID_OV, RG_NA_OV);
  new T_RG_PLAY;
  new T_RG_BACK;
  new T_RG_FORW;
  new T_RG_1STEP_B;
  new T_RG_1STEP_F;
  new T_RG_HOME;
  new T_RG_END;
  new T_RG_QUIT;
  new T_RG_SCUP;
  new T_RG_SCDOWN;
  new T_RG_EDUP;
  new T_RG_EDDOWN;
  new T_RG_OPEN_MD;
  new T_RG_O_MDLST;
  new T_RG_S_MDLST;
  new T_RG_C_MDLST;
  new T_RG_CUT_ED;
  new T_RG_PASTE_ED;
  new T_RG_TRIM_L;
  new T_RG_TRIM_R;
  new T_RG_TRIM_SW;
  new T_RG_CL_GOP;
  new T_RG_OPEN_PRJ;
  new T_RG_SAVE_PRJ;
  new T_RG_COPY_PRJ;
  new T_RG_PLAYSL;
  new T_RG_GROW_L;
  new T_RG_GROW_R;
  new T_RG_SHRINK_L;
  new T_RG_SHRINK_R;
  new T_RG_AVS_L;
  new T_RG_AVS_R;
  new T_RG_TEXT(RG_ID_TEXT_PRJ, RG_NA_TEXT_PRJ);
  new T_RG_TEXT(RG_ID_TEXT_MD, RG_NA_TEXT_MD);
  new T_RG_TEXT(RG_ID_TEXT_PGMD, RG_NA_TEXT_PGMD);
  new T_RG_TEXT(RG_ID_TEXT_PGED, RG_NA_TEXT_PGED);
  new T_RG_TEXT(RG_ID_TEXT_AVS, RG_NA_TEXT_AVS);
  
  //-- Table regions --
  new T_RG_MEDIA(RG_ID_MEDIA);
  new T_RG_EDIT(RG_ID_EDIT);
}

//----------------------------------------------------------------------------

int T_RG_PLAY::OnMouseLeftDown(int num)
{  
  if (ctx->action != A_NOMEDIA)
  {
    T_RG_TOGGLE::OnMouseLeftDown(num);

    ctx->iv = IV_NORM;
    if (ctx->action != A_PLAY)
      SwitchOn(num);
    else
      SwitchOff(num);
  
    if (rg_focus) rg_focus->MemberPage();
  }
  return RG_NOKEY;
}  

void T_RG_PLAY::SwitchOff(int num)
{
  T_RG_TOGGLE::SwitchOff(num);
  if (ctx->action != A_NOMEDIA) 
  {
    //-- show slider --
    ((T_RG_PLAYSL &)REGION(RG_ID_PLAYSL)).UnHide();
    ctx->action = A_STOP;
  }
}

void T_RG_PLAY::SwitchOn(int num)
{
  T_RG_TOGGLE::SwitchOn(num);
  
  if (ctx->action != A_NOMEDIA) 
  {  
    //-- hide slider --
    ((T_RG_PLAYSL &)REGION(RG_ID_PLAYSL)).Hide();
    ctx->action = A_PLAY;
  }
}

//----------------------------------------------------------------------------

int T_RG_QUIT::OnMouseLeftUp(int num)
{  
  T_RG_TOGGLE::OnMouseLeftUp(num);
  return ksym;
}  

//----------------------------------------------------------------------------

int T_RG_BACK::OnShiftMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);
  
  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->act_real - 1;  // will work for field pictures too !
    int l = 2;
    
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    for (int i=0; i<l; i++) p = ctx->file->prevSequence(p - 1);
    
    if (p>0) 
      ctx->step = -(ctx->act_real-p);
    else
      ctx->step = -24;
       
    ctx->act_frame += ctx->step;   
    ctx->adj_pos    = 1;
  }  
  return RG_NOKEY;
}
  
int T_RG_BACK::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);
  
  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->act_real - 1; // will work for field pictures too !
   
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    if ( (p = ctx->file->prevSequence(p-1)) > 0 )
      ctx->step = -(ctx->act_real-p);
    else
      ctx->step = -12;  
    
    ctx->act_frame += ctx->step;   
    ctx->adj_pos    = 1;
     
  }  
  return RG_NOKEY;
}  

int T_RG_BACK::OnCtrlMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);
  
  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->act_real - 1;  // will work for field pictures too !
    int l = 4 * step_multi;
    
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    for (int i=0; i<l; i++) p = ctx->file->prevSequence(p - 1);
      
    if (p>0) 
      ctx->step = -(ctx->act_real-p);
    else
      ctx->step = -(step_multi * 48);
      
    ctx->act_frame += ctx->step;   
    ctx->adj_pos    = 1;
  }  
  return RG_NOKEY;
}  

  
//----------------------------------------------------------------------------

int T_RG_FORW::OnShiftMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->file->nextSequence(ctx->act_real + 1);
    p = ctx->file->nextSequence(p + 1);
    
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    ctx->step       = (p - ctx->act_real);
    ctx->act_frame += ctx->step;
    ctx->adj_pos    = 1;
  }
  return RG_NOKEY;
}  

int T_RG_FORW::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->file->nextSequence(ctx->act_real + 1);
     
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    ctx->step       = (p - ctx->act_real);
    ctx->act_frame += ctx->step;
    ctx->adj_pos    = 1;
  }
  return RG_NOKEY;
}  

int T_RG_FORW::OnCtrlMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    int p = ctx->act_real;
    int l = 4 * step_multi;
    
    for (int i=0; i<l; i++) p = ctx->file->nextSequence(p + 1);
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    ctx->step       = (p - ctx->act_real);
    ctx->act_frame += ctx->step;
    ctx->adj_pos    = 1;
  }
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_1STEP_B::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    ctx->step       = -1;
    ctx->act_frame -= 1;
    ctx->run_once   = 1;
    ctx->adj_pos    = 1;
  }
  return RG_NOKEY;
}  
  
int T_RG_1STEP_B::OnShiftMouseLeftDown(int num)
{
  ctx->act_frame--;
  return OnMouseLeftDown(num);
}  
  
int T_RG_1STEP_B::OnCtrlMouseLeftDown(int num)
{
  ctx->act_frame -= 5;
  return OnMouseLeftDown(num);
}  
  
//----------------------------------------------------------------------------

int T_RG_1STEP_F::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    if (ctx->action == A_PLAY) rg_get(RG_ID_PLAY).OnMouseLeft(num);
    ctx->run_once = 1;
  }
  else
    ctx->adj_pos = 0;
    
  return RG_NOKEY;
}  

int T_RG_1STEP_F::OnShiftMouseLeftDown(int num)
{
  ctx->act_frame += 2;
  ctx->adj_pos = 1;
  return OnMouseLeftDown(num);
}  
  
int T_RG_1STEP_F::OnCtrlMouseLeftDown(int num)
{
  ctx->act_frame += 4;
  ctx->adj_pos = 1;
  return OnMouseLeftDown(num);
}  

//----------------------------------------------------------------------------

int T_RG_HOME::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);
  
  if (ctx->action != A_NOMEDIA)
  {
    if (ctx->action == A_STOP) ctx->run_once = 1;
    
    if (!trim_mode)
      ctx->act_frame = 0;
    else
      ctx->act_frame = ctx->start;  
    ctx->adj_pos   = 1;
  }  
  return RG_NOKEY;
}  
  
//----------------------------------------------------------------------------

int T_RG_END::OnMouseLeftDown(int num)
{  
  T_RG_DOWN::OnMouseLeftDown(num);

  if (ctx->action != A_NOMEDIA)
  {
    if (!trim_mode)
      ctx->act_frame = ctx->dist_sum - 1;
    else
      ctx->act_frame = ctx->end;
    
    ctx->run_once  = 1;
    ctx->adj_pos   = 1;
  }
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_SCUP::OnMouseLeftUp(int num)
{  
  T_RG_DOWN::OnMouseLeftUp(num);

  RG_MEDIA.PrevPage(); 
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_SCDOWN::OnMouseLeftUp(int num)
{  
  T_RG_DOWN::OnMouseLeftUp(num);

  RG_MEDIA.NextPage(); 
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_EDUP::OnMouseLeftUp(int num)
{  
  T_RG_DOWN::OnMouseLeftUp(num);

  RG_EDIT.PrevPage(); 
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_EDDOWN::OnMouseLeftUp(int num)
{  
  T_RG_DOWN::OnMouseLeftUp(num);

  RG_EDIT.NextPage(); 
  return RG_NOKEY;
}  

//----------------------------------------------------------------------------

int T_RG_OPEN_MD::OnMouseLeftUp(int num)
{
  char *fname = NULL;
  
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);

  fname = lve_fselect_media(QDIR_OPEN, QDIR_MD);
  
  if ((fname != NULL) && (*fname != ' '))
  {
    RG_MEDIA.LoadMedia(fname, 0, 0);
  }
  
  return RG_NOKEY;
}  

int T_RG_OPEN_MD::OnShiftMouseLeftUp(int num)
{
  return 0;
}  

//----------------------------------------------------------------------------

int T_RG_O_MDLST::OnMouseLeftUp(int num)
{
  T_LST_EDIT *ed_lst = NULL;
  char       *fname  = NULL;
  
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
 
  fname = lve_fselect_media(QDIR_NEW, QDIR_MD_LST);
  if ( fname )
  {
    if (RG_MEDIA.GetList()->LoadMembers(fname))
      RG_MEDIA.rg_single_focus(0,1);
    else 
    {
      ed_lst = (T_LST_EDIT *)RG_EDIT.GetList();
      if (ed_lst)
      {
        if (ed_lst->MemberCnt()) 
          RG_EDIT.rg_single_focus(0,1);
        else
          ctx->action = A_NOMEDIA;
      }
      else
        ctx->action = A_NOMEDIA;
    }
        
    RG_TEXT(RG_ID_TEXT_MD).Print(fname);
    RG_MEDIA.DisplayUpdate();
  }
  
  return RG_NOKEY;
} 

int T_RG_S_MDLST::OnMouseLeftUp(int num)
{
  char *fname = NULL;
 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
 
  fname = RG_TEXT(RG_ID_TEXT_MD).Get();
  if (!strlen(fname))
  {
    fname = lve_fselect_media(QDIR_SAVE, QDIR_MD_LST);
    if (fname) RG_TEXT(RG_ID_TEXT_MD).Print(fname);
  }  
    
  if (fname) RG_MEDIA.GetList()->SaveMembers(fname);
  
  return RG_NOKEY;
} 

int T_RG_C_MDLST::OnMouseLeftUp(int num)
{
  char *fname = NULL;
 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
 
  fname = lve_fselect_media(QDIR_SAVE, QDIR_MD_LST);
  if ( fname )
  {
    RG_MEDIA.GetList()->SaveMembers(fname);
    RG_TEXT(RG_ID_TEXT_MD).Print(fname);
  }
  return RG_NOKEY;
} 

//----------------------------------------------------------------------------

int T_RG_CUT_ED::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);

  if (RG_EDIT.GetList()->GetFocusMember()) RG_EDIT.CutVideo();
} 

//----------------------------------------------------------------------------

int T_RG_PASTE_ED::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
  
  //-- insert --
  if (RG_EDIT.GetList()->GetFocusMember()) RG_EDIT.PasteVideo(0);
} 

int T_RG_PASTE_ED::OnShiftMouseLeftDown(int num)
{
  return T_RG_DOWN::OnMouseLeftDown(num);
}

int T_RG_PASTE_ED::OnShiftMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
  
  //-- append --
  if (RG_EDIT.GetList()->GetFocusMember()) RG_EDIT.PasteVideo(1);

  return T_RG_DOWN::OnMouseLeftUp(num);
} 

//----------------------------------------------------------------------------

static char avs_str[16];

int T_RG_AVS_L::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  
  ctx->avs_dp--;
  if (ctx->avs_dp < -1000) ctx->avs_dp = -1000;
  lve_set_avs(ctx->avs_dp);
  
  return RG_NOKEY;
} 

int T_RG_AVS_L::OnShiftMouseLeftUp(int num)
{ 
  ctx->avs_dp -= 9;
  return OnMouseLeftUp(num);
} 

int T_RG_AVS_R::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  
  ctx->avs_dp++;
  if (ctx->avs_dp > 1000) ctx->avs_dp = 1000;
  lve_set_avs(ctx->avs_dp);
  
  return RG_NOKEY;
} 

int T_RG_AVS_R::OnShiftMouseLeftUp(int num)
{ 
  ctx->avs_dp += 9;
  return OnMouseLeftUp(num);
} 


//----------------------------------------------------------------------------

int T_RG_TRIM_L::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);

  RG_EDIT.TrimVideo(0, draw_thumb);

  return RG_NOKEY;
} 

int T_RG_TRIM_L::IsSensitive(int num)
{
  if ( RG_EDIT.GetList()->MemberCnt() )
    return trim_mode;
  else
    return 0;
}

int T_RG_SHRINK_L::OnMouseLeftUp(int num)
{
  ctx->act_real = ctx->file->nextSequence(ctx->start_real+1);
  T_RG_TRIM_L::OnMouseLeftUp(num);
  
  ctx->adj_pos   = 1;
  ctx->run_once  = 1;  // force jump to new pos ...
  ctx->act_real -= 1;  // ... but prevent scene rollover
  ctx->need_thumb = 2; // also (edit-list!) thumbnail update needed 
  
  return RG_NOKEY;
}

int T_RG_GROW_L::OnMouseLeftUp(int num)
{
  if ( ctx->start_real > 1 ) 
  {  
    ctx->act_real = ctx->file->prevSequence(ctx->start_real - 1);
    T_RG_TRIM_L::OnMouseLeftUp(num);
  
    ctx->run_once   = 1;  // force jump to new pos
    ctx->adj_pos    = 1;
    ctx->need_thumb = 2;  // also (edit-list!) thumbnail update needed
  }
  return RG_NOKEY;
}


//----------------------------------------------------------------------------

int T_RG_TRIM_R::OnMouseLeftUp(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
  
  RG_EDIT.TrimVideo(1);

  return RG_NOKEY;
} 

int T_RG_TRIM_R::IsSensitive(int num)
{
  if ( RG_EDIT.GetList()->MemberCnt() )
    return trim_mode;
  else
    return 0;
}

int T_RG_SHRINK_R::OnMouseLeftUp(int num)
{
  ctx->act_real = ctx->file->prevSequence(ctx->end_real - 1) - 1;
  T_RG_TRIM_R::OnMouseLeftUp(num);
  
  ctx->adj_pos   = 1;
  ctx->run_once  = 1;  // force jump to new pos ...
  ctx->act_real -= 1;  // ... but prevent scene rollover
  
  return RG_NOKEY;
}

int T_RG_GROW_R::OnMouseLeftUp(int num)
{
  ctx->act_real = ctx->file->nextSequence(ctx->end_real + 3) - 1;
  
  T_RG_TRIM_R::OnMouseLeftUp(num);
  
  ctx->adj_pos   = 1;
  ctx->run_once  = 1;  // force jump to new pos ...
  ctx->act_real -= 1;  // ... but prevent scene rollover
  
  return RG_NOKEY;
}

//----------------------------------------------------------------------------

int T_RG_TRIM_SW::OnMouseLeftDown(int num)
{ 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);

  T_RG_TOGGLE::OnMouseLeftDown(num);
  trim_mode = !trim_mode;
} 

void T_RG_TRIM_SW::SwitchOff(int num)
{
  T_RG_TOGGLE::SwitchOff(num);
  trim_mode = 0;
}

void T_RG_TRIM_SW::SwitchOn(int num)
{
  T_RG_TOGGLE::SwitchOn(num);
  trim_mode = 1;
}

//----------------------------------------------------------------------------

int T_RG_CL_GOP::OnMouseLeftDown(int num)
{ 
  char txt[8];
  
  T_RG_DOWN::OnMouseLeftDown(num);
  step_multi++;
  if (step_multi > 3) step_multi = 1;
  
  sprintf(txt, "%dx", step_multi);
  DT_DrawText(txt, ctx->screen, 0, GetRectX(0)+55, GetRectY(0)+1, 0);
  ctx->refresh_bg = 1;
} 

//----------------------------------------------------------------------------

int T_RG_OPEN_PRJ::OnMouseLeftUp(int num)
{
  T_LST_MEDIA *md_lst = NULL;
  char        *fname  = NULL;
  
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
 
  fname = lve_fselect_media(QDIR_NEW, QDIR_ED_PRJ);
  if ( fname )
  {
    if (RG_EDIT.GetList()->LoadMembers(fname))
      RG_EDIT.rg_single_focus(0,1);
    else 
    {
      md_lst = (T_LST_MEDIA *)RG_MEDIA.GetList();
      if (md_lst)
      {
        if (md_lst->MemberCnt()) 
          RG_MEDIA.rg_single_focus(0,1);
        else
          ctx->action = A_NOMEDIA;
      }
      else
        ctx->action = A_NOMEDIA;
    }
    
    RG_TEXT(RG_ID_TEXT_PRJ).Print(fname);
    RG_EDIT.DisplayUpdate();
  }
  return RG_NOKEY;
} 

int T_RG_SAVE_PRJ::OnMouseLeftUp(int num)
{
  char *fname = NULL;
 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
  fname = RG_TEXT(RG_ID_TEXT_PRJ).Get();

  //-- fname not known -> start file selector --
  if (!strlen(fname))
  {
    fname = lve_fselect_media(QDIR_SAVE, QDIR_ED_PRJ);
    if (fname) RG_TEXT(RG_ID_TEXT_PRJ).Print(fname);
  }  
  
  if (fname) RG_EDIT.GetList()->SaveMembers(fname);
  
  return RG_NOKEY;
} 

int T_RG_COPY_PRJ::OnMouseLeftUp(int num)
{
  char *fname = NULL;
 
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
    
  T_RG_DOWN::OnMouseLeftUp(num);
 
  fname = lve_fselect_media(QDIR_SAVE, QDIR_ED_PRJ);
    
  if ( fname )
  {
    RG_EDIT.GetList()->SaveMembers(fname);
    RG_TEXT(RG_ID_TEXT_PRJ).Print(fname);
  }
  return RG_NOKEY;
} 

//----------------------------------------------------------------------------

void T_RG_PLAYSL::Hide(void)
{
  return;
  /*
  sl_hidden = 1;
  rg_draw(0, 0);
  */
}

void T_RG_PLAYSL::UnHide(void)
{
  //sl_hidden = 0;
  state = 0;
  Update();
}

int T_RG_PLAYSL::Update(void)
{
  static int old_x = -1;
  
  int    x, dx;
  int    act_frame;
  int    range;
  int    start, end; 
 
  if (ctx->action == A_NOMEDIA) return 0;
  
  if (trim_mode)
  {
    start = ctx->start;
    end   = ctx->end;
    range = ctx->end - ctx->start + 1;
  }
  else
  {
    start = 0;
    end   = ctx->dist_sum - 1;
    range = ctx->dist_sum;
  }  
  
  //-- left mouse key not pressed --
  if (state == 0) 
  {
    x = (int)(sl_min_x + sl_w * (ctx->act_frame - start)/(double)range);
    if (old_x != x) 
    {
      if (x < sl_min_x) x = sl_min_x;
      else if (x > sl_max_x) x = sl_max_x;
     
      rg_draw(1, x - sl_w_half);
      old_x = x;
    }  
    return 0;
  }

  SDL_GetMouseState(&x, NULL);
  
  dx = (x-old_x);
  if (!dx) return 0;
  
  if (x < sl_min_x) 
  {
     act_frame = start;
     x         = sl_min_x;
  }  
  else if (x > sl_max_x)
  {
    act_frame = end;
    x         = sl_max_x;
  }
  else  
  {
    int gop_aligned = 0;
    int dp          = (int)( (x - sl_min_x) / (double)sl_w * range );
    
    //-- faster slider movement may be GOP accurate ... --
    //----------------------------------------------------
    if ( abs(dx) > 4 ) // && trim_mode )
    {
      int p1 = ctx->start_real + dp;
      int p2;
      
      if (dx > 0 && p1 < ctx->end_real)
      {
        if ( (p2 = ctx->file->prevSequence(p1)) > ctx->act_real )
        {
          gop_aligned = 1;
          act_frame   = (p2 - ctx->start_real) + start;
        }
      }  
      else
      {
        if ( (p2 = ctx->file->nextSequence(p1)) < ctx->act_real )
        { 
          gop_aligned = 1;
          act_frame = (p2 - ctx->start_real) + start;
        }
      }
    }
    
    if (!gop_aligned)
    {
      act_frame = start + dp;
      if (act_frame > end ) act_frame = end;
    }
  }
  
  old_x = x;
  
  rg_draw(1, x - sl_w_half);
  
  ctx->act_frame = act_frame;
  ctx->run_once  = 1;
  ctx->adj_pos   = 1;
 
  return 1;
}

int T_RG_PLAYSL::OnMouseLeftDown(int num)
{
  if (ctx->action != A_NOMEDIA)
  {
    REGION(RG_ID_PLAY).SwitchOff(0);

    //-- left mouse key pressed --
    T_RG_SLIDER::OnMouseLeftDown(num);
    state = 1;

    ctx->action = A_SLIDE;
  }
  return RG_NOKEY;
}

int T_RG_PLAYSL::OnMouseLeftUp(int num)
{
  state = 0;

  if (ctx->action != A_NOMEDIA)
  {
    ctx->action   = A_STOP;
    ctx->adj_pos  = 1;
    ctx->run_once = 1;
  }
}

void T_RG_PLAYSL::OnCommonUp()
{
  state = 0;
  if (ctx->action != A_NOMEDIA) ctx->action = A_STOP;
}

//----------------------------------------------------------------------------

void T_RG_MEDIA::PrintPageNum(void)
{
  char pnum[128];
  
  sprintf(pnum, "%d/%d", md_list.GetPageNum(), md_list.GetNPages());
  RG_TEXT(RG_ID_TEXT_PGMD).Print(pnum);
}

int T_RG_MEDIA::OnMouseLeftDown(int num)
{
  //-- stop video --
  REGION(RG_ID_PLAY).SwitchOff(0);  
  return T_RG_DRAG::OnMouseLeftDown(num);
}

int T_RG_MEDIA::IsSensitive(int num)
{
  return (md_list.GetMember(num) != NULL);
}

void T_RG_MEDIA::OnChangeFocus(int num)
{
  //-- request position update (seeking to members start_frame !) --
  ctx->adj_pos = 3;
}  

void T_RG_MEDIA::OnFocus(int num)
{
  if (ctx->action == A_NOMEDIA) return;
  
  ctx->dist_sum = md_list.FramesCnt();
  if (ctx->action == A_STOP) ctx->run_once = 1;
}  

int T_RG_MEDIA::GetPos(void)
{
  T_MEMBER   *mb_start = NULL;
  T_MB_MEDIA *mb       = (T_MB_MEDIA *)md_list.GetFocusMember();
  T_MB_MEDIA *mb_sav   = mb;
  int        found     = 0;
  
  do 
  {
    if ( ctx->act_frame >= mb->GetStartFrame() )
    {
      if ( ctx->act_frame <= mb->GetEndFrame() )
      {
        found = 1;
        break;   
      }
    }  
  } while ( (mb = (T_MB_MEDIA *)md_list.GetNextMember(mb)) != NULL );
  
  if (!found)
  {
    mb = mb_sav;
    do 
    {
      if ( ctx->act_frame <= mb->GetEndFrame() ) 
      {
        if ( ctx->act_frame >= mb->GetStartFrame() )
        {
          found = 1;
          break;
        } 
      }
    } while ( (mb = (T_MB_MEDIA *)md_list.GetPrevMember(mb)) != NULL );
  }
  
  if (!found)
  {  
    fprintf(stderr, "WARNING: T_RG_MEDIA::GetPos() - out of range !\n");
    fprintf(stderr, "act (%d), start/end (%d/%d)\n", ctx->act_frame, ctx->start, ctx->end); 
    fprintf(stderr, "jumping to start position\n");
    ctx->act_frame = 0;
    return 0;
  }
  
  //-- if researched member isn't current --
  if (mb != md_list.GetFocusMember()) 
  {
    //-- adjust list ready to display selected member --
    mb_start = md_list.MemberPage(mb);
    
    //-- now set focus with display-num of selected member ! --
    rg_single_focus(md_list.MemberDisplayNum(mb), (mb_start != NULL));
    
    //-- refresh display list --
    DisplayUpdate(mb_start);

    //-- no further run after automatic launched  "rg_single_focus" !!! --
    ctx->run_once = 0;
  }   
  return mb->GetRealFrame(); 
}      

void T_RG_MEDIA::SplitVideo(void)
{
  T_MB_MEDIA *mb;
  
  //-- cut only, if this region has focus --  
  if (rg_focus == this)
  {
    //-- work on focus member --
    if ( (mb = (T_MB_MEDIA *)md_list.GetFocusMember()) != NULL)
    {
      if ( (md_list.InsertMember(mb->Split(), mb)) != NULL)
      {
        int n;
        
        if (rg_focus_num+1 >= RectCnt()) 
        {
          md_list.NextPage();  
          n = 0;
        }
        else
          n = rg_focus_num + 1;  

        //-- select this --
        rg_single_focus(n, 1);
        
        //-- request for new thumbnail with display update --
        md_list.GetFocusMember()->GenThumb();
        DisplayUpdate(NULL);
      }
    }  
  }
}
    
void T_RG_MEDIA::JoinVideo(void)
{
  T_MB_MEDIA *mb, *mb_first, *mb_last;
  int        id, gap, n;
  
  //-- cut only, if this region has focus --  
  gap = 2;
  if (rg_focus == this)
  {
    //-- start with focus member --
    if ( (mb = (T_MB_MEDIA *)md_list.GetFocusMember()) != NULL)
    {
      mb_first = mb;
      id       = mb->GetMDFId();
      gap      = 0;
      while (mb)
      {
        if (mb->HasMark()) 
        {
          mb_last = mb;
          if (mb->GetMDFId() != id)
          {
            gap = 2;
            fprintf(stderr, "WARNING: T_RGMEDIA::JoinVideo() - marked range contains different media\n");
            break;
          }
          
          if (gap)
          {
            gap = 2;
            fprintf(stderr, "WARNING: T_RGMEDIA::JoinVideo() - marked range has gaps\n");
            break;
          }
        }
        else    
          gap = 1;
          
        mb = (T_MB_MEDIA *)mb->GetNext();
      }
    }
    
    if (gap != 2)
    {
      mb = mb_last;
      mb_first->Join(mb);
      while (mb != mb_first)
      {
        mb = (T_MB_MEDIA *)mb_last->GetPrev();
        md_list.RemoveMember(mb_last);
        mb_last = mb;
      }
      mb_first->OnSelect();
      
      //-- ... then refresh Display --
      DisplayUpdate(md_list.GetStartMember());
    }
  }  
}

int T_RG_MEDIA::LoadMediaList(char *name)
{
  int rc;
  
  REGION(RG_ID_PLAY).SwitchOff(0);
  ctx->action = A_NOMEDIA; 

  if ( (GetList()->LoadMembers(name)) != 0)
  { 
    //-- select the new scene --
    rg_single_focus(0,1);
    DisplayUpdate();
        
    //-- switch to normal timer freq. --
    ctx->iv = IV_NORM; 
    return 1;
  }
  
  return 0;
}

void T_RG_MEDIA::LoadMedia(char *name, int a_stream, int v_stream)
{
  int          media_id;
  T_MEMBER*    mb;
  T_LST_EDIT  *ed_lst = NULL;
  T_LST_MEDIA *md_lst = NULL;
  
  REGION(RG_ID_PLAY).SwitchOff(0);
 
  display_OvText(ctx, "  ", 0x05);
  display_OvText(ctx, "  CREATING INDEX,", 0x00);
  display_OvText(ctx, "  PLEASE WAIT ...", 0x02);
  display_refresh(ctx);
 
  //-- create new media scene member --
  if ( (mb = md_list.LoadMedia(name, a_stream, v_stream)) == NULL ) 
  {
    display_OvText(ctx, " ", 0x05);
    display_OvText(ctx, "ERROR loading file:", 0x00);
    display_OvText(ctx, name, 0x02);
    display_refresh(ctx);
      
    return;
  }
      
  //-- jump to last display page --
  md_list.LastPage();
  
  //-- select the new scene --
  OnMouseLeft(md_list.MemberPageCnt()-1);
  
  //-- request for new thumbnail with display  --
  //-- update and switch to normal timer freq. --
  ctx->need_thumb = 1;
  ctx->run_once   = 2;
  ctx->iv         = IV_NORM; 
}    

int T_RG_MEDIA::rg_invert(int new_flag, int num, T_MEMBER* mb)
{
  if (new_flag == pflag[num]) return 0;
  pflag[num] = new_flag;
 
  if (dst_rect)
  {
    Uint32 color;
    
    //-- fill complete box with color --
    color = SDL_MapRGB(ctx->screen->format, 0xFF, 0xFF, 0x06);
    SDL_FillRect(ctx->screen, &dst_rect[num], color);
    
    //-- redraw thumbnail inside of box (now there is a clored border :-) -- 
    if (mb) mb->BlitThumb(&dst_rect[num]);
    
    //-- need refresh of display --
    ctx->refresh_bg = 1;
  }
  
  return 0;
}                 


//----------------------------------------------------------------------------

void T_RG_EDIT::PrintPageNum(void)
{
  char pnum[128];
  
  sprintf(pnum, "%d/%d", ed_list.GetPageNum(), ed_list.GetNPages());
  RG_TEXT(RG_ID_TEXT_PGED).Print(pnum);
}

void T_RG_EDIT::TrimVideo(int side, int draw_thumb)
{
  T_MB_EDIT *mb = (T_MB_EDIT *)ed_list.GetFocusMember();
  
  if (ed_list.MemberDisplayNum(mb) != -1) 
  {
    ed_list.TrimMember(mb, side, draw_thumb);

    //-- correct scene parameters without seeking (!) --   
    ctx->adj_pos = 2; 
    mb->OnSelect();
    
    //-- move slider to pos --
    if (side) 
      ctx->act_frame = mb->GetEndFrame();
    else
      ctx->act_frame = mb->GetStartFrame();
         
    ((T_RG_PLAYSL &)REGION(RG_ID_PLAYSL)).Update();
  }  
  DisplayUpdate();
}

void T_RG_EDIT::CutVideo(void)
{
  T_MB_EDIT *mb_cut, *mb_sav = NULL;
  int       n, start = 0;
  
  //-- for all marked members of edit-list --
  mb_cut  = (T_MB_EDIT *)ed_list.GetFirstMember();
  
  //-- reset clipboard --
  cl_list.Reset();
   
  while (mb_cut)
  {
    if (mb_cut->HasMark())
    {
      mb_sav = (T_MB_EDIT *)mb_cut->GetNext();
      
      //-- remove member from edit-list without deleting object ... --
      ed_list.RemoveMember(mb_cut, 0);
      
      //-- ... needed to append on clipboard list -- 
      cl_list.AppendMember(mb_cut);
      
      mb_cut = mb_sav;
    }  
    else
      mb_cut = (T_MB_EDIT *)mb_cut->GetNext();
  }        
  
  //-- adjust focus -> maybe in media-list --
  if (ed_list.MemberCnt() == 0)
  {
    ClearMark(0);
    
    //-- any member there ? --
    if (RG_MEDIA.GetList()->MemberCnt() > 0)
    {
      RG_MEDIA.rg_single_focus(0, 1);
      RG_MEDIA.DisplayUpdate(); 
    }
    else
      ctx->action = A_NOMEDIA;
  }
  else
  {
    //-- update display of edit list --
    if (mb_sav)
      n = ed_list.MemberDisplayNum(mb_sav);
    else
      n = ed_list.MemberPageCnt()-1;
 
    if (n < 0) n = 0;
    rg_single_focus(n, 1);

    DisplayUpdate(ed_list.GetStartMember());
  }
}

void T_RG_EDIT::PasteVideo(int append)
{
  T_MB_EDIT   *mb_edit, *mb_dup, *mb_clip;
  int         i, n;

  //-- members in clipord ? --
  if (cl_list.MemberCnt() ==  0) return;
  
  if (append)
  {
    mb_edit = NULL;
    n       = ed_list.MemberPageCnt();
  }
  else
  {
    mb_edit = (T_MB_EDIT*)ed_list.GetFocusMember();
    n       = ed_list.MemberDisplayNum(mb_edit);
  }
     
  //-- for all marked members of source list --
  mb_clip = (T_MB_EDIT *)cl_list.GetFirstMember();
  while (mb_clip)
  {
    if ( (mb_dup = (T_MB_EDIT *)mb_clip->Dup()) != NULL)
    {
      if (append)
      {
        ed_list.AppendMember(mb_dup);
      }
      else
        ed_list.InsertMember(mb_dup, mb_edit);
    }  
    mb_clip = (T_MB_EDIT *)mb_clip->GetNext();
  }
    
  //-- select this --
  rg_single_focus(n, 1);
  DisplayUpdate();
}  

int T_RG_EDIT::OnMouseLeftDown(int num)
{
  //-- stop video --
  REGION(RG_ID_PLAY).SwitchOff(0);  

  lve_decoder_switch_lock(1);
  return T_RG_DRAGNDROP::OnMouseLeftDown(num);
}

int T_RG_EDIT::OnMouseLeftUp(int num)
{
  lve_decoder_switch_lock(0);
  return T_RG_DRAGNDROP::OnMouseLeftUp(num);
}

void T_RG_EDIT::OnMouseCommonUp(void)
{
  lve_decoder_switch_lock(0);
  T_RG_DRAGNDROP::OnMouseCommonUp();
}

int T_RG_EDIT::IsSelectable(int num)
{
  return (ed_list.GetMember(num) != NULL);
}

void T_RG_EDIT::OnDrop(int num, char mv_flag)
{
  T_LST_SCENE *src_list = (T_LST_SCENE *)rg_drag->GetList();
  T_MB_SCENE  *mb_src;
  T_MB_SCENE  *mb_dst;
  T_MB_SCENE  *mb_dup;
  int         i, n;
  int         insert = 0;

  //-- drop only, if source has list --  
  if (src_list)
  {
    //-- get mode (insert / append) --
    n = GetList()->MemberPageCnt();
    
    switch(SDL_GetModState())
    {
      case KMOD_LCTRL:
      case KMOD_RCTRL:
      case KMOD_LSHIFT:
      case KMOD_RSHIFT:
        break; // append
        
      default:   
        if ((n > 0) && (num < n))
        {
          //-- get actual member to insert bevor --
          mb_dst = (T_MB_SCENE *)GetList()->GetMember(num);
          insert = 1;
          if (mv_flag && (num>0) ) num--; 
        }
        else
        { 
          //-- adjust number -> append --   
          num = n;
        }
        break;
    }
     
    //-- for all marked members of source list --
    mb_src = (T_MB_SCENE *)src_list->GetFirstMember();
    
    //-- special handling --
    if ( insert && (src_list == GetList()) )
    {
      char snap = 0;
      
      mb_dup = NULL;
      while(mb_src)
      {
        if ( mb_src->HasMark() )
        {
          //-- remember 1st. marked --
          if (!mb_dup) mb_dup = mb_src;
          
          if (mb_src == mb_dst) 
            snap = 1;
        }
        else if (mb_src == mb_dst)
          snap = 0; 
        
        mb_src = (T_MB_SCENE *)mb_src->GetNext();  
      }
      
      //-- restore on 1st. marked --
      if (snap) 
      {
        fprintf(stderr, "drop inside selection not possible !\n");
        mb_dup = NULL;
      }
      mb_src = mb_dup;
    }
    
    while (mb_src)
    {
      if (mb_src->HasMark())
      {
        if ( (mb_dup = mb_src->Dup()) != NULL)
        {
          if (insert)
            GetList()->InsertMember(mb_dup, mb_dst);
          else
            GetList()->AppendMember(mb_dup);
        }
        mb_dup = mb_src;  
      } 
      else
        mb_dup = NULL;
        
      mb_src = (T_MB_SCENE *)mb_src->GetNext();
      if (mv_flag && mb_dup) 
      {  
        src_list->RemoveMember(mb_dup);
        if (!insert) num--;
      }  
    }
    
    //-- select this --
    rg_single_focus(num, 1);
    DisplayUpdate();
  }
  else
    fprintf(stderr, "WARNING: T_RG_EDIT::OnDrop() - source region has no list\n"); 
}  

void T_RG_EDIT::OnChangeFocus(int num)
{
  //-- request position update (seeking to members start_frame !) --
  ctx->adj_pos = 3;
}  

void T_RG_EDIT::OnFocus(int num)
{
  ctx->dist_sum = ed_list.FramesCnt();
  if (ctx->action == A_STOP) ctx->run_once = 1;
}  

int T_RG_EDIT::GetPos(void)
{
  //int        num;
  T_MEMBER   *mb_start = NULL;
  T_MB_EDIT  *mb       = (T_MB_EDIT *)ed_list.GetFocusMember();
  T_MB_EDIT  *mb_sav   = mb;
  int        found     = 0;
  
  do 
  {
    if ( ctx->act_frame >= mb->GetStartFrame() )
    {
      if ( ctx->act_frame <= mb->GetEndFrame() )
      {
        found = 1;
        break;   
      }
    }  
  } while ( (mb = (T_MB_EDIT *)ed_list.GetNextMember(mb)) != NULL );
  
  if (!found)
  {
    mb = mb_sav;
    do 
    {
      if ( ctx->act_frame <= mb->GetEndFrame() ) 
      {
        
        if ( ctx->act_frame >= mb->GetStartFrame() )
        {
          found = 1;
          break;
        } 
      }
    } while ( (mb = (T_MB_EDIT *)ed_list.GetPrevMember(mb)) != NULL );
  }
  
  if (!found)
  {  
    fprintf(stderr, "WARNING: T_RG_EDIT::GetPos() - out of range !\n");
    fprintf(stderr, "act (%d), start/end (%d/%d)\n", ctx->act_frame, ctx->start, ctx->end); 
    fprintf(stderr, "jumping to start position\n");
    ctx->act_frame = 0;
    return 0;
  }
  
  //-- if researched member isn't current one --
  if (mb != ed_list.GetFocusMember())
  {
    //-- adjust list to display selected member --
    mb_start = ed_list.MemberPage(mb);
    
    //-- now set focus with  display-num of selected member ! --
    rg_single_focus(ed_list.MemberDisplayNum(mb), (mb_start != NULL));
    
    //-- refresh display list --
    DisplayUpdate(mb_start);

    //-- no further run after automatic launched  "rg_single_focus" !!! --
    ctx->run_once = 0;
  } 
  return mb->GetRealFrame();
}      

int T_RG_EDIT::LoadEditList(char *name)
{
  REGION(RG_ID_PLAY).SwitchOff(0);
  SDL_Delay(100);
  
  if ( (GetList()->LoadMembers(name)) != 0)
  {
    //-- select the first edit-scene (force focus) --
    rg_single_focus(0, 1);
    DisplayUpdate();
  
    return 1;
  }
  
  return 0;
}  

int T_RG_EDIT::rg_invert(int new_flag, int num, T_MEMBER* mb)
{
  if (new_flag == pflag[num]) return 0;
  pflag[num] = new_flag;
 
  if (dst_rect)
  {
    Uint32 color;
    
    //-- fill complete box with color --
    color = SDL_MapRGB(ctx->screen->format, 0xFF, 0xFF, 0x06);
    SDL_FillRect(ctx->screen, &dst_rect[num], color);
    
    //-- redraw thumbnail inside of box (now there is a colored border :-) -- 
    if (mb) mb->BlitThumb(&dst_rect[num]);
    
    //-- need refresh of display --
    ctx->refresh_bg = 1;
  }
  
  return 0;
}                 
