/*
 *  lve_gui.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifndef _LVE_GUI_H
#define _LVE_GUI_H 1
#include "lve.h"
#include "region.h"
#include "media.h"
#include "edit.h"

//=====================================================
//== definition of ids and names for the own regions ==
//=====================================================
#define RG_NA_OV        "rg_overlay"
#define RG_NA_PLAY	"rg_play"
#define RG_NA_FORW	"rg_forw"
#define RG_NA_BACK	"rg_back"
#define RG_NA_1STEP_F	"rg_1step_f"
#define RG_NA_1STEP_B   "rg_1step_b"
#define RG_NA_HOME	"rg_home"
#define RG_NA_END	"rg_end"
#define RG_NA_QUIT	"rg_quit"
#define RG_NA_MEDIA	"rg_scene"
#define RG_NA_EDIT	"rg_edit"
#define RG_NA_PLAYSL	"rg_playsl"
#define RG_NA_SCUP	"rg_scup"
#define RG_NA_SCDOWN	"rg_scdown"
#define RG_NA_EDUP	"rg_edup"
#define RG_NA_EDDOWN	"rg_eddown"
#define RG_NA_OPEN_MD   "rg_open_md"
#define RG_NA_O_MDLST   "rg_o_mdlst"
#define RG_NA_S_MDLST   "rg_s_mdlst"
#define RG_NA_C_MDLST   "rg_c_mdlst"
#define RG_NA_OPEN_PRJ	"rg_open_prj"
#define RG_NA_SAVE_PRJ	"rg_save_prj"
#define RG_NA_COPY_PRJ	"rg_copy_prj"
#define RG_NA_CUT_ED	"rg_cut_ed"
#define RG_NA_TRIM_L	"rg_trim_l"
#define RG_NA_TRIM_R	"rg_trim_r"
#define RG_NA_TRIM_SW   "rg_trim_sw"
#define RG_NA_PASTE_ED  "rg_paste_ed"
#define RG_NA_TEXT_PRJ  "rg_text_prj"
#define RG_NA_TEXT_MD   "rg_text_md"
#define RG_NA_TEXT_PGMD "rg_text_pgmd"
#define RG_NA_TEXT_PGED "rg_text_pged"
#define RG_NA_CL_GOP    "rg_cl_gop"
#define RG_NA_GROW_R    "rg_grow_r"
#define RG_NA_GROW_L	"rg_grow_l"
#define RG_NA_SHRINK_R	"rg_shrink_r"
#define RG_NA_SHRINK_L	"rg_shrink_l"
#define RG_NA_AVS_L	"rg_avs_l"
#define RG_NA_AVS_R	"rg_avs_r"
#define RG_NA_TEXT_AVS	"rg_text_avs"

// guess "#define RG_ID_UNDEF	0" is made in region.h

#define RG_ID_OV	1
#define RG_ID_PLAY	2
#define RG_ID_FORW	3
#define RG_ID_BACK	4
#define RG_ID_1STEP_F	5
#define RG_ID_1STEP_B   6
#define RG_ID_HOME	7
#define RG_ID_END	8
#define RG_ID_QUIT	9
#define RG_ID_MEDIA    10
#define RG_ID_EDIT     11
#define RG_ID_PLAYSL   12
#define RG_ID_SCUP     13
#define RG_ID_SCDOWN   14
#define RG_ID_EDUP     15
#define RG_ID_EDDOWN   16
#define RG_ID_OPEN_MD  17
#define RG_ID_O_MDLST  18
#define RG_ID_S_MDLST  19
#define RG_ID_C_MDLST  20
#define RG_ID_CUT_ED   21
#define RG_ID_TRIM_L   22
#define RG_ID_TRIM_R   23
#define RG_ID_TRIM_SW  24
#define RG_ID_OPEN_PRJ 25
#define RG_ID_SAVE_PRJ 26
#define RG_ID_COPY_PRJ 27
#define RG_ID_PASTE_ED 28	
#define RG_ID_TEXT_PRJ 29
#define RG_ID_TEXT_MD  30
#define RG_ID_TEXT_PGMD 31
#define RG_ID_TEXT_PGED 32
#define RG_ID_CL_GOP    33
#define RG_ID_GROW_R	34
#define RG_ID_GROW_L	35
#define RG_ID_SHRINK_R	36
#define RG_ID_SHRINK_L	37
#define RG_ID_AVS_L     38
#define RG_ID_AVS_R     39
#define RG_ID_TEXT_AVS  40 

//=====================
//== GUI-Application ==
//=====================
class T_APP
{
  public:
    T_APP(T_LVE_CTX *ctx, char *lib_path);

    void CreateRegions(void);
};

//========================================
//== screen regions for GUI-Application ==
//========================================

class T_RG_PLAY : public T_RG_TOGGLE
{
  public:
    virtual int  OnMouseLeftDown(int num);
    virtual void SwitchOff(int num);
    virtual void SwitchOn(int num);
    
    T_RG_PLAY::T_RG_PLAY(void) : T_RG_TOGGLE(RG_ID_PLAY, RG_NA_PLAY, SDLK_p)
    {
    };
};

class T_RG_BACK : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num);
    
    T_RG_BACK::T_RG_BACK(void) : T_RG_DOWN(RG_ID_BACK, RG_NA_BACK, SDLK_PAGEDOWN)
    {
    };
};


class T_RG_FORW : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num);
    
    T_RG_FORW::T_RG_FORW(void) : T_RG_DOWN(RG_ID_FORW, RG_NA_FORW, SDLK_PAGEUP)
    {
    };
};

class T_RG_1STEP_B : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num); 
    
    T_RG_1STEP_B::T_RG_1STEP_B(void) : T_RG_DOWN(RG_ID_1STEP_B, RG_NA_1STEP_B, SDLK_DOWN)
    {
    };
};

class T_RG_1STEP_F : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num); 

    T_RG_1STEP_F::T_RG_1STEP_F(void) : T_RG_DOWN(RG_ID_1STEP_F, RG_NA_1STEP_F, SDLK_UP)
    {
    };
};

class T_RG_HOME : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    
    T_RG_HOME::T_RG_HOME(void) : T_RG_DOWN(RG_ID_HOME, RG_NA_HOME, SDLK_HOME)
    {
    };
};


class T_RG_END : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
    
    T_RG_END::T_RG_END(void) : T_RG_DOWN(RG_ID_END, RG_NA_END, SDLK_END)
    {
    };
};

class T_RG_QUIT : public T_RG_TOGGLE
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_QUIT::T_RG_QUIT(void) : T_RG_TOGGLE(RG_ID_QUIT, RG_NA_QUIT, SDLK_q)
    {
    };
};

class T_RG_SCUP : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_SCUP::T_RG_SCUP(void) : T_RG_DOWN(RG_ID_SCUP, RG_NA_SCUP, RG_NOKEY)
    {
    };
};

class T_RG_SCDOWN : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_SCDOWN::T_RG_SCDOWN(void) : T_RG_DOWN(RG_ID_SCDOWN, RG_NA_SCDOWN, RG_NOKEY)
    {
    };
};

class T_RG_EDUP : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_EDUP::T_RG_EDUP(void) : T_RG_DOWN(RG_ID_EDUP, RG_NA_EDUP, RG_NOKEY)
    {
    };
};

class T_RG_EDDOWN : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_EDDOWN::T_RG_EDDOWN(void) : T_RG_DOWN(RG_ID_EDDOWN, RG_NA_EDDOWN, RG_NOKEY)
    {
    };
};

class T_RG_OPEN_MD : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int OnShiftMouseLeftDown(int num) { return OnMouseLeftDown(num); };
    virtual int OnShiftMouseLeftUp(int num);
    
    T_RG_OPEN_MD::T_RG_OPEN_MD(void) : T_RG_DOWN(RG_ID_OPEN_MD, RG_NA_OPEN_MD, RG_NOKEY)
    {
    };
};

class T_RG_O_MDLST : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_O_MDLST::T_RG_O_MDLST(void) : T_RG_DOWN(RG_ID_O_MDLST, RG_NA_O_MDLST, RG_NOKEY)
    {
    };
};

class T_RG_S_MDLST : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_S_MDLST::T_RG_S_MDLST(void) : T_RG_DOWN(RG_ID_S_MDLST, RG_NA_S_MDLST, RG_NOKEY)
    {
    };
};

class T_RG_C_MDLST : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_C_MDLST::T_RG_C_MDLST(void) : T_RG_DOWN(RG_ID_C_MDLST, RG_NA_C_MDLST, RG_NOKEY)
    {
    };
};

class T_RG_CUT_ED : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_CUT_ED::T_RG_CUT_ED(void) : T_RG_DOWN(RG_ID_CUT_ED, RG_NA_CUT_ED, RG_NOKEY)
    {
    };
};

class T_RG_PASTE_ED : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnShiftMouseLeftUp(int num);
    
    T_RG_PASTE_ED::T_RG_PASTE_ED(void) : T_RG_DOWN(RG_ID_PASTE_ED, RG_NA_PASTE_ED, RG_NOKEY)
    {
    };
};

class T_RG_TRIM_SW : public T_RG_TOGGLE
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual void SwitchOff(int num);
    virtual void SwitchOn(int num);
       
    T_RG_TRIM_SW::T_RG_TRIM_SW(void) : T_RG_TOGGLE(RG_ID_TRIM_SW, RG_NA_TRIM_SW, RG_NOKEY)
    {
    };
};

//------------------------------------------------------------------------------

class T_RG_TRIM_L : public T_RG_DOWN
{
  protected:
    int draw_thumb;
    
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int IsSensitive(int num);
    
    T_RG_TRIM_L::T_RG_TRIM_L(int id, const char *name, int ksym) 
      : T_RG_DOWN(id, name, ksym)
    {
      draw_thumb = 0;
    };
    
    T_RG_TRIM_L::T_RG_TRIM_L(void) 
      : T_RG_DOWN(RG_ID_TRIM_L, RG_NA_TRIM_L, RG_NOKEY)
    {
      draw_thumb = 1;
    };
};

class T_RG_SHRINK_L : public T_RG_TRIM_L
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_SHRINK_L::T_RG_SHRINK_L(void) 
      : T_RG_TRIM_L(RG_ID_SHRINK_L, RG_NA_SHRINK_L, RG_NOKEY)
    {
    };
};

class T_RG_GROW_L : public T_RG_TRIM_L
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_GROW_L::T_RG_GROW_L(void) 
      : T_RG_TRIM_L(RG_ID_GROW_L, RG_NA_GROW_L, RG_NOKEY)
    {
    };
};

class T_RG_TRIM_R : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int IsSensitive(int num);
    
    T_RG_TRIM_R::T_RG_TRIM_R(int id, const char *name, int ksym) 
      : T_RG_DOWN(id, name, ksym)
    {
    };
    
    T_RG_TRIM_R::T_RG_TRIM_R(void) 
      : T_RG_DOWN(RG_ID_TRIM_R, RG_NA_TRIM_R, RG_NOKEY)
    {
    };
};

class T_RG_SHRINK_R : public T_RG_TRIM_R
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_SHRINK_R::T_RG_SHRINK_R(void) 
      : T_RG_TRIM_R(RG_ID_SHRINK_R, RG_NA_SHRINK_R, RG_NOKEY)
    {
    };
};

class T_RG_GROW_R : public T_RG_TRIM_R
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_GROW_R::T_RG_GROW_R(void) 
      : T_RG_TRIM_R(RG_ID_GROW_R, RG_NA_GROW_R, RG_NOKEY)
    {
    };
};

//------------------------------------------------------------------------------

class T_RG_AVS_L : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int OnShiftMouseLeftUp(int num);
    
    T_RG_AVS_L::T_RG_AVS_L(void) : T_RG_DOWN(RG_ID_AVS_L, RG_NA_AVS_L, RG_NOKEY)
    {
    };
    
};

class T_RG_AVS_R : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    virtual int OnShiftMouseLeftUp(int num);
    
    T_RG_AVS_R::T_RG_AVS_R(void) : T_RG_DOWN(RG_ID_AVS_R, RG_NA_AVS_R, RG_NOKEY)
    {
    };
};

//------------------------------------------------------------------------------

class T_RG_OPEN_PRJ : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_OPEN_PRJ::T_RG_OPEN_PRJ(void) : T_RG_DOWN(RG_ID_OPEN_PRJ, RG_NA_OPEN_PRJ, RG_NOKEY)
    {
    };
};

class T_RG_SAVE_PRJ : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_SAVE_PRJ::T_RG_SAVE_PRJ(void) : T_RG_DOWN(RG_ID_SAVE_PRJ, RG_NA_SAVE_PRJ, RG_NOKEY)
    {
    };
};

class T_RG_COPY_PRJ : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftUp(int num);
    
    T_RG_COPY_PRJ::T_RG_COPY_PRJ(void) : T_RG_DOWN(RG_ID_COPY_PRJ, RG_NA_COPY_PRJ, RG_NOKEY)
    {
    };
};

class T_RG_CL_GOP : public T_RG_DOWN
{
  public:
    virtual int OnMouseLeftDown(int num);
       
    T_RG_CL_GOP::T_RG_CL_GOP(void) : T_RG_DOWN(RG_ID_CL_GOP, RG_NA_CL_GOP, SDLK_m)
    {
    };
};

class T_RG_PLAYSL : T_RG_SLIDER
{
  private:
    int      state;

  public:
    T_RG_PLAYSL::T_RG_PLAYSL() 
      : T_RG_SLIDER(RG_ID_PLAYSL, RG_NA_PLAYSL)
    {
      state     = 0;
      sl_hidden = 0;
    };
    
    virtual int  OnMouseLeftDown(int num);
    virtual int  OnMouseLeftUp(int num);
    virtual void OnCommonUp(void);
    
    void Hide(void);
    void UnHide(void);
    int  Update(void);
};

#define RG_PLAYSL ((T_RG_PLAYSL &)REGION(RG_ID_PLAYSL))

//============================================================================

class T_RG_MEDIA : public T_RG_DRAG
{
  protected:
    T_LST_MEDIA   md_list;
    
  public:
    T_RG_MEDIA::T_RG_MEDIA(int id) 
                          : T_RG_DRAG(id, RG_NA_MEDIA, RG_NOKEY, 5, 3)
    {
      md_list.Setup(ctx, RectCnt());
      
      //-- guess to set the default list-pointer --
      //-- it is very imortant for the automtic  --
      //-- marking of the data list members      -- 
      SetList(&md_list);
    };
    
    virtual int    rg_invert(int new_flag, int num, T_MEMBER* mb = NULL);

    virtual void   OnFocus(int num);
    virtual void   OnChangeFocus(int num);
    virtual int    GetPos(void);
    virtual int    IsSensitive(int num);
    virtual void   PrintPageNum(void);
    virtual int    OnMouseLeftDown(int num);
    
    void SplitVideo(void);
    void JoinVideo(void);
    void LoadMedia(char *name, int a_stream, int v_stream);
    int  LoadMediaList(char *name);
};

#define RG_MEDIA ((T_RG_MEDIA &)REGION(RG_ID_MEDIA))


//============================================================================

class T_RG_EDIT : public T_RG_DRAGNDROP
{
  protected:
    T_LST_EDIT ed_list;
    T_LIST     cl_list;  // clipboard !
    
  public:
    T_RG_EDIT::T_RG_EDIT(int id) 
                        : T_RG_DRAGNDROP(id, RG_NA_EDIT, RG_NOKEY, 9, 2)
    {
      ed_list.Setup(ctx, RectCnt());
      cl_list.Setup(ctx, 1);
      
      //-- guess to set the default list-pointer --
      //-- it is very imortant for the automtic  --
      //-- marking of the data list members      -- 
      SetList(&ed_list);
    };
 
    virtual int    rg_invert(int new_flag, int num, T_MEMBER* mb = NULL);

    virtual void   OnDrop(int num, char mv_flag);
    virtual void   OnFocus(int num);
    virtual void   OnChangeFocus(int num);
    virtual int    GetPos(void);

    virtual int    OnMouseLeftDown(int num);
    virtual int    OnMouseLeftUp(int num);
    virtual void   OnMouseCommonUp(void);
    virtual int    IsSelectable(int num);
    virtual void   PrintPageNum(void); 
    
    void CutVideo(void);
    void TrimVideo(int side, int draw_thumb = 1);
    void PasteVideo(int append);
    int  LoadEditList(char *name);
};

#define RG_EDIT ((T_RG_EDIT &)REGION(RG_ID_EDIT))

//============================================================================

class T_RG_LVE : public T_REGION
{
  public:
    virtual void rg_create_regions(void);
};

#endif // _PLAYER_H 
