/*
 *  lve_read.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifndef _LVE_READ_H
#define _LVE_READ_H
#include <stdio.h>
#include <inttypes.h>
#include "media_buf.h"

#define SC_EOF        0
#define SC_NORM       1

typedef struct
{
  void *fs;
  int  media_type;

  int  new_scene;
  int  total_scenes;
  int  total_frames;
    
  int      src_w;
  int      src_h;
  int      pic_size_l;
  int      pic_size_c;

  double   frame_rate;
  int      aspect;

  int      total_audio;
  int      has_audio;
  int	   sample_rate;
  int      channels;

  uint8_t  *src_buf;
  uint8_t  *py;
  uint8_t  *pu; 
  uint8_t  *pv;
  
  int      samples;
  double   samples_cnt[3];
  double   pcm_scale; 
  int      aframe_size;
  
  char     *lst_fname; 
  int      read_mode;  
  int      v_stream;
  int      a_stream;
  int      force_atrack;

  uint8_t     *chunk_buf;
  int      buf_len;

  int         act_real;
  T_MEDIA_HDR md_header;
  T_MEDIA_BUF md_buf;
  T_MEDIA_BUF *md_bup;
  
  int         v_bitrate;
  int         a_bitrate;
  
  int         a_shift;
  int         encoded_fs;
  
} T_LVE_READ_CTX;


#ifdef __cplusplus
extern "C"
{
#endif

extern int            lr_file_chk(char *fname);
extern T_LVE_READ_CTX *lr_init(int a_track, int a_shift);
extern void           lr_cleanup(void);

extern int   lr_init_wav(FILE *afp, int wav_sr);
extern int   lr_get_achunk(T_LVE_READ_CTX *pctx);
extern int   lr_get_vchunk(T_LVE_READ_CTX *pctx, int p, int gop_bound, void *newParms);
extern int   lr_get_frame(T_LVE_READ_CTX *pctx);
extern int   lr_get_samples(T_LVE_READ_CTX *pctx, uint8_t *pbuf, int force_pal);
extern int   lr_resample(T_LVE_READ_CTX *pctx, int dst_sr, int init);
extern int   lr_media_chk_range(T_LVE_READ_CTX *pctx, int force_next);
extern int   lr_acodec_id(void);

extern uint8_t *lr_mSeqHdr(int *len);

#ifdef __cplusplus
}
#endif

#endif // _LVE_READ_H
