/*
 *  lvemkdvd.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "media_buf.h"
#include "lve_config.h"

#define MD_LVEDUMP    0 
#define MD_LVEMUX     1
#define MD_TCMPLEX    2
#define MD_MPLEX      3
#define MD_SHRINK_DMP 4
#define MD_SHRINK_MUX 5

#define FN_XML "lvemkdvd.xml"


int main(int argc, char **argv)
{
  struct stat stat_buf;
  int         i, j, l, m_cnt;

  int    mode           = MD_LVEMUX;
  double f              = 1.0;
  int    only_size_info = 0;
  FILE   *xml_fp        = NULL;
  char   *mux_opts      = "";
  char 	 *author_opts   = "";
  char   _dvd_dir[3]    = "./";
  char   *dvd_dir       = _dvd_dir;
  char   *media[256];
  char   cmd_line[1024];
  
  //-- get command line options --
  //------------------------------
  m_cnt = 0;
  for (i=1; i<argc; i++)
  {
    //-- output directory --
    if (!strcmp("-o", argv[i]))
    {  
      i++;
      if (i<argc) dvd_dir = argv[i];
    }
    //-- optional mux parameter list delimited by ':' (!) --
    //-- or embedded in inside a "-pair                   --
    else if (!strcmp("-m", argv[i]))
    {  
      if (++i<argc) 
      {
        mux_opts = argv[i];
        for (j=0; j<strlen(mux_opts); j++)
        {
          if (mux_opts[j] == ':') mux_opts[j] = ' ';
        }
      }    
    }
    //-- optional dvdauthor parameter list delimited by ":" (!) --
    //-- or embedded in inside a "-pair                         --
    else if (!strcmp("-i", argv[i]))
    {  
      i++;
      if (i<argc) 
      {
        author_opts = argv[i];
        for (j=0; j<strlen(author_opts); j++)
        {
          if (author_opts[j] == ':') author_opts[j] = ' ';
        }
      }
    }
    else if ( !strcmp("-f", argv[i]) )
    {
      if (++i < argc) 
      {
        f = atof(argv[i]);
        if (f<1.0) f = 1.0;
      }
    }
    else if (!strcmp("-x", argv[i]))
    {
      mode = MD_TCMPLEX;
    }  
    else if (!strcmp("-y", argv[i]))
    {
      mode = MD_MPLEX;
    }  
    else if (!strcmp("-p", argv[i]))
    {
      mode = MD_LVEDUMP;
    }
    else if (!strcmp("-P", argv[i]))
    {
      mode = MD_LVEDUMP;
      only_size_info = 1;
    }    
    //-- media file pairs --
    else
    {
      media[m_cnt] = argv[i];
      m_cnt++;
    }
  }
  
  //-- check all input files --
  //---------------------------
  if ( mode == MD_LVEMUX || mode == MD_TCMPLEX || mode == MD_MPLEX )
  {
    char *av_name = cmd_line;
    
    for (i=0; i<m_cnt; i++)
    {
      strcpy(av_name, media[i]);
      strcat(av_name, ".m2v");
      if (stat(av_name, &stat_buf) == -1)
      {
        sprintf(av_name, "ERROR - (%s.m2v)", media[i]);
        perror(av_name);
        return 0;
      } 
    
      strcpy(av_name, media[i]);
      strcat(av_name, ".mpa");
      if (stat(av_name, &stat_buf) == -1)
      {
        sprintf(av_name, "ERROR - (%s.mpa)", media[i]);
        perror(av_name);
        return 0;
      }
    }
  }
  else
  {
    T_MEDIA_HDR hdr;
    FILE        *cfp;
    off_t       total_size_mb = 0;
      
    for (i=0; i<m_cnt; i++)
    {
      if (!strchr(media[i], '.')) strcat(media[i], ".prj");
      
      if ( (cfp = fopen(media[i], "rb")) == NULL || !fread(&hdr, sizeof(T_MEDIA_HDR), 1, cfp) ) 
      {
        sprintf(cmd_line, "ERROR - (%s)", media[i]);
        perror(media[i]);
        if (cfp) fclose(cfp);
        return 0;
      } 
      
      if ( strncmp(hdr.magic, EDIT_BUF_MAGIC, MAGIC_LEN) )
      {
        fprintf(stderr, "ERROR - (%s) is no edit-list !\n", media[i]);
        fclose(cfp);
        return 0;
      }  
      
      fclose(cfp);
      total_size_mb += hdr.output_size_mb;
    }
    
    fprintf(stderr, "NOTE: edit-list mode enabled -> estimated total size (%ld MB)\n", total_size_mb);
    
    if (only_size_info) return 0;
  }
  
  //-- not enough args --
  //---------------------
  if (!m_cnt) 
  {
    fprintf(stderr, 
      "usage: lvemkdvd [options] <s1> [<s2> ... <sN>]\n"
      "options are:\n"
      " -p enables the \"edit-list\" mode to use edit-lists as input files\n"
      " -P prints estimated total size of dvd image in \"edit-list\" mode and exit\n"
      " -m <par> to bypass additional options <par> to lvemux/lvedump\n"  
      " -i <par> to bypass additional options <par> to dvdauthor\n" 
      " -o <dir> to write to  destination folder <dir> instead using ./\n"
      " -x invokes tcmplex instead of lvemux for multiplexing\n"
      " -y invokes mplex instead of lvemux for multiplexing\n"
      "\n"
      "  <s1>, <s2> ... <sN> are common names of source media A/V-file pairs\n"
      "  (eg. common name of s1.m2v s1.mpa is s1) or edit-list files (-p opt.)\n" 
    );
    return 0;
  }     

  //-- check for m2v requantizer use --
  //-----------------------------------
  if (f != 1.0) switch(mode)
  {
    case MD_LVEDUMP:
      if ( stat("/tmp/shrink.m2v", &stat_buf) == -1 )
        system("mkfifo /tmp/shrink.m2v /tmp/shrink.mpa");
      mode = MD_SHRINK_DMP;
      break; 

    case MD_LVEMUX:
      mode = MD_SHRINK_MUX;
      break;
      
    default:
      f = 1.0;  
      break;  
  }
  
  //-- check (create) fifo --
  //-------------------------
  if ( stat("/tmp/mux", &stat_buf) != -1 )
  {
    if (!S_ISFIFO(stat_buf.st_mode))
    {
      fprintf(stderr, "ERROR: /tmp/mux: already exists but is no FIFO!\n"); 
      return 0;
    }  
  }
  else
    system("mkfifo /tmp/mux");
  
  strcpy(cmd_line, lc_getWorkDir());
  strcat(cmd_line, FN_XML);
  if ( (xml_fp = fopen(cmd_line,"w")) == NULL )
  {
    fprintf(stderr, "ERROR: could not create (%s)\n", FN_XML); 
    return 0;
  }
  
  
  
  //-- constant xml-header --
  //-------------------------   
  fputs("<dvdauthor>\n", xml_fp); 
  fputs("  <vmgm />\n", xml_fp);
  fputs("  <titleset>\n", xml_fp);
  fputs("    <titles>\n", xml_fp);
  fputs("      <pgc>\n", xml_fp);  
  
  //-- several vob lines (as sub command) --
  //----------------------------------------
  for (i=0; i<m_cnt; i++) 
  { 
    switch(mode)
    { 
      case MD_LVEDUMP:
        sprintf
        ( 
          cmd_line, 
          "        <vob file=\"lvedump %s -m -s 48000 -i %s -av - 2>/dev/null|\" />\n", 
          mux_opts, media[i]
        );
        break; 
      
      case MD_LVEMUX:
        sprintf
        (
          cmd_line, 
          "         <vob file=\"lvemux %s -v %s.m2v -a %s.mpa -o - 2>/dev/null|\" />\n", 
          mux_opts, media[i], media[i]
        );
        break;
        
      case MD_TCMPLEX:
        sprintf
        (
          cmd_line, 
          "         <vob file=\"tcmplex -m d %s -i %s.m2v -p %s.mpa -o /tmp/mux 2>/dev/null | cat /tmp/mux |\" />\n", 
          mux_opts, media[i], media[i]
        );
        break; 
      
      case MD_MPLEX:
        sprintf
        (
          cmd_line, 
          "         <vob file=\"mplex -f 8 %s -o /tmp/mux %s.m2v %s.mpa 2>/dev/null | cat /tmp/mux |\" />\n", 
          mux_opts, media[i], media[i]
        );
        break; 
        
      case MD_SHRINK_DMP:
        sprintf
        (
          cmd_line,
          "        <vob file=\"lvedump %s -e -s 48000 -i %s -av /tmp/shrink 2>/dev/null | lverequant %1.5f /tmp/shrink.m2v /tmp/mux | lvemux -p -v /tmp/mux -a /tmp/shrink.mpa -o - 2>/dev/null|\" />\n", 
          mux_opts, media[i], f
        );  
        break;
        
      case MD_SHRINK_MUX:
        sprintf
        (
          cmd_line, 
          "         <vob file=\"lverequant %1.5f %s.m2v /tmp/mux | lvemux %s -v /tmp/mux -a %s.mpa -o - 2>/dev/null|\" />\n", 
          f, media[i], mux_opts, media[i]
        );
        break;
         
    }
    fputs(cmd_line, xml_fp);
  }
  
  //-- constant xml-footer --
  //-------------------------
  fputs("      </pgc>\n", xml_fp);  
  fputs("    </titles>\n", xml_fp);
  fputs("  </titleset>\n", xml_fp);
  fputs("</dvdauthor>\n", xml_fp); 
  
  fclose(xml_fp);
  
  //-- build dvdauthor-command for xml input ... --
  //-----------------------------------------------
  sprintf(cmd_line, "dvdauthor %s -o %s -x %s%s", author_opts, dvd_dir, lc_getWorkDir(), FN_XML);

  
  //-- ... and launch task --
  //-------------------------
  fprintf(stderr,"launch: %s\n", cmd_line);
  system(cmd_line);
  fprintf(stderr,"lvemkdvd done !\n");
  
  return 1;
}

// eof
