/*
 *  lvemkidx.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <stdio.h>
#include <unistd.h>

#include "decoder.h"
#include "audio.h"
#include "media_buf.h"

static T_USER_PROGRESS myProgress;

int main(int argc, char **argv)
{
  int  i, dmy;
  FILE *f_prj;
  int  probe_only = 0;
  char *idx_name  = NULL;
  char *prj_name  = NULL;
  int  maxIndex   = -1;
  
  for (i=1; i<argc; i++)
  {
    if (!strcmp("-i", argv[i]))
    {  
      i++;
      if (i<argc) idx_name = argv[i];
    }
    else if (!strcmp("-l", argv[i]))
    {  
      i++;
      if (i<argc) prj_name = argv[i];
    }
    else if (!strcmp("-f", argv[i]))
    {  
      i++;
      if (i<argc) maxIndex = atol(argv[i]);
    }
    else if (!strcmp("-p", argv[i]))
    {
      probe_only = 1;
    }
    else if (!strcmp("-g", argv[i]))
    {
      T_FRAME_SRV::setUserProgress(&myProgress);
    }
  }
    
  if ( idx_name )
  {
    T_FRAME_SRV *fs = T_FRAME_SRV::createFrameSrv(idx_name);
    T_MPEG_DEC  decoder;
 
    if (fs)
    {
      int t;
      
      //-- select maximum --
      fs->setMaxIndex(maxIndex);
      
      switch( t=fs->getType() )
      {
        case MPEG_LVE:
          if (prj_name) printf("%d\n", t); // print type 
          fprintf(stderr, "INFO: (%s) is a LVE project file, no index needed !\n", idx_name);
          break;
          
        case MPEG_ES:
          if (prj_name)
          {
            struct stat st; 
            char *chk_name = strdup(idx_name);
            int  l         = strlen(chk_name) - 4;
            
            if (l>=0) chk_name[l] = '\0';
            strcat(chk_name, ".mpa");
            
            if ( stat(chk_name, &st) == -1 ) 
            {
              if (l>=0) chk_name[l] = '\0';
              strcat(chk_name, ".wav");
              
              if ( stat(chk_name, &st) != -1 ) 
              {
                t = MPEG_WA;
              }
              else
              {
                fprintf(stderr, "INFO: no correspondend audio file (mpa/wav) found !");
                t = MPEG_NA;
              }
            }
            else
            {
              //-- ES stream with separate audio file doesn't need index --
              printf("%d\n", t);
              break;
            }  
          }
          // no break !
          
        case MPEG_PS:
        case MPEG_TS:
        case MPEG_PES:
        case MPEG_PVA:
          if ( !probe_only ) decoder.forceIndex();
          if ( decoder.open(fs, &dmy) ) 
          {
            int    size;
            int    maxSize  = 0;
            int    minSize  = 0x7FFFFFFF;
            double median   = 0;
            double br25Min  = 1000000;
            double br25Max  = 0;
            double br250Min = 1000000;
            double br250Max = 0;
            
            for (i=1; i<fs->nFrames(); i++)
            {
              size = fs->filePos(i) - fs->filePos(i-1);
              
              //-- total --
              if (size < minSize) 
                minSize = size;
              else if (size > maxSize)
                maxSize = size;
                
              median = (median*(i-1) + size) / (double)i;
              
              //-- per second --
              if (i>=24) 
              {
                double br25 = (fs->filePos(i)-fs->filePos(i-24))*8/1024.0; 
                
                if (br25 < br25Min) br25Min = br25;
                else if (br25 > br25Max) br25Max = br25;
              }
              
              //-- per 10 seconds --
              if (i>=249) 
              {
                double br250 = (fs->filePos(i)-fs->filePos(i-249))*8/10240.0; 
                
                if (br250 < br250Min) br250Min = br250;
                else if (br250 > br250Max) br250Max = br250;
              }
              
            }  
          
            fprintf
            (
              stderr, 
              "\npictures sizes:\n"
              "  min: %d\n"
              "  max: %d\n"
              "  avg: %1.1f\n"
              "\nbitrate statistics:\n"
              "  total: %1.1f KBit/s\n",
              minSize, maxSize, median, (median * 25 * 8) / 1024
            );

            if ( fs->nFrames() > 25 )
            {
              fprintf
              ( 
                stderr, 
                "  min over 1s:  %1.1f KBit/s\n"
                "  max over 1s:  %1.1f KBit/s\n",
                br25Min, br25Max
              );
            }
           
            if ( fs->nFrames() > 250 )
            {
              fprintf
              (
                stderr,
                "  min over 10s: %1.1f KBit/s\n"
                "  max over 10s: %1.1f KBit/s\n",
                br250Min, br250Max
              );
            }                
          
            fprintf(stderr, "\n");
            
            if (prj_name) 
            {
              if ( (f_prj = fopen(prj_name,"w+b")) != NULL )
              {
                T_MEDIA_HDR hdr;
                T_MEDIA_BUF en;
                struct stat st;
                
                printf("%d\n", t);  // print type
                
                //-- build header --
                //------------------
                strcpy(hdr.magic, EDIT_BUF_MAGIC);
                hdr.mb_cnt   = 1;
                hdr.dist_sum = fs->nFrames() - 1;
                
                if (stat(idx_name, &st) != -1)
                  hdr.output_size_mb = st.st_size / (1024 * 1024);
                else
                  hdr.output_size_mb = 0;
                  
                //-- write header --
                //------------------
                fwrite(&hdr, 1, sizeof(T_MEDIA_HDR), f_prj);
                
                //-- build single entry --
                //------------------------
                en.media_id   = 0;
                en.vstream    = 0;
                en.astream    = 0;
                en.start_real = 0;
                en.end_real   = hdr.dist_sum-1;
                en.dist       = en.end_real;
                en.start      = 0;
                en.end        = en.end_real;
                en.avs_dp     = 0;
                en.fname[0]   = '\0';
 	      	
                if (*idx_name != '/') 
                {
                  //strcpy(en.fname, get_current_dir_name());
                  getcwd(en.fname, 1000);
                  strcat(en.fname,"/");
                }  
                strcat(en.fname, idx_name);
                memset(en.bmp_thumb, 0xFF, BMP_TH_LEN);
                
                //-- write entry --
                //-----------------
                fwrite(&en, 1, sizeof(T_MEDIA_BUF), f_prj);
                
                fclose(f_prj);
              }
              else
              {
                printf("-1\n");
                fprintf(stderr, "ERR: couldn't create LVE project-file (%s)\n", prj_name);
              }
            }
            decoder.close(fs);
          }
          break;  
          
        default:
          if (prj_name) printf("-1\n");
          break;  
          
      }
    }
  }
  else
    fprintf(stderr,"usage: lvemkidx [-f <frames>] [-p] [-l <prj-file>] -i <mpeg-file>\n");
}

