/*
 *  media.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include "media.h"

T_MB_SCENE *T_MB_MEDIA::Dup(void)
{
  T_MB_EDIT *mb_dup;
  mb_dup = new T_MB_EDIT(ctx, 0);
  
  if (mb_dup)     
  {
    mb_dup->mdf         = mdf->dup();
    mb_dup->astream     = astream;
    mb_dup->vstream     = vstream;
    
    mb_dup->dist        = dist;
    mb_dup->start_real  = start_real;
    mb_dup->end_real    = end_real;
    mb_dup->start       = start;
    mb_dup->end         = end;
    mb_dup->avs_dp      = avs_dp;
    
    memcpy(mb_dup->bmp_thumb, bmp_thumb, BMP_TH_LEN);
    
    mb_dup->has_mark    = 0;

    return mb_dup;
  }  

  fprintf(stderr, "ERROR: T_MB_MEDIA::Dup() - out of memory on dup !\n"); 
  return NULL;
}

T_MB_MEDIA *T_MB_MEDIA::Split(void)
{
  T_MB_MEDIA *mb_dup;
  
  if ( (ctx->act_frame == ctx->start) ||
       (ctx->act_frame == ctx->end) )
  {
    fprintf(stderr, "WARNING: T_MB_MEDIA::Dup() - dup on boundary not possible !\n"); 
    return NULL;
  }
  
  mb_dup = new T_MB_MEDIA(ctx, mb_num);
  if (mb_dup)     
  {
    mb_dup->mdf         = mdf->dup();
    mb_dup->astream     = astream;
    mb_dup->vstream     = vstream; 

    mb_dup->start_real  = start_real;
    mb_dup->end_real    = ctx->act_real - 1;
    mb_dup->dist        = ctx->act_real - start_real;
    mb_dup->start       = start;
    mb_dup->end         = start + mb_dup->dist - 1;
    mb_dup->avs_dp      = avs_dp;
    mb_dup->has_mark    = 0;
    memcpy(mb_dup->bmp_thumb, bmp_thumb, BMP_TH_LEN);

    start_real  = ctx->act_real;
    dist        = end_real - start_real + 1;
    start       = end - dist + 1;

    has_mark    = 0;
   
    return mb_dup;
  }  

  fprintf(stderr, "ERROR: T_MB_MEDIA::Split() - out of memory on dup !\n"); 
  return NULL;
}

void T_MB_MEDIA::Join(T_MB_SCENE *mb)
{
  if (mb)     
  {
    end_real = mb->end_real;
    dist     = mb->dist + dist;
    end      = mb->end;
  }  
}

void T_MB_MEDIA::Save(FILE *lst_file)
{
  T_MEDIA_BUF media_buf;

  media_buf.media_id     = mdf->getId();
  media_buf.vstream      = vstream;
  
  media_buf.astream      = ctx->a_stream; // astream; 
  media_buf.start        = start;
  media_buf.end          = end;
  media_buf.dist         = dist;
  media_buf.start_real   = start_real;
  media_buf.end_real     = end_real;
  media_buf.avs_dp       = avs_dp;
  strcpy(media_buf.fname, mdf->getName());
  memcpy(media_buf.bmp_thumb, bmp_thumb, BMP_TH_LEN);  
  
  fwrite(&media_buf, sizeof(T_MEDIA_BUF), 1, lst_file);
}

int T_MB_MEDIA::Load(FILE *lst_file)
{
  T_MEDIA_BUF media_buf;
  TMediaFile  *_mdf;
  int         n = 0;
  
  n = fread(&media_buf, sizeof(T_MEDIA_BUF), 1, lst_file);
  
  if (n != 0)
  {
    _mdf = TMediaFile::createMDF(media_buf.fname);
    if (_mdf == NULL) return -1;
    
    if (!Setup(_mdf, media_buf.astream, media_buf.vstream) ) 
    {   
      TMediaFile::releaseMDF(_mdf);
      mdf = NULL;
      return -1;
    }             

    start         = media_buf.start;
    end           = media_buf.end;
    dist          = media_buf.dist;
    start_real    = media_buf.start_real;
    end_real      = media_buf.end_real;
    avs_dp        = media_buf.avs_dp;
    
    memcpy(bmp_thumb, media_buf.bmp_thumb,  BMP_TH_LEN);  
  }

  return (n==0);
}

//=============================================================

void T_LST_MEDIA::Reset(void)
{
  T_LIST::Reset();
 
  dist_sum = 0;
  mb_cnt   = 0;
  // never reset the media-id !!!
}

T_MEMBER* T_LST_MEDIA::LoadMedia(char *fname, int _astream, int _vstream)
{
  T_MB_MEDIA *mb = NULL;
  
  if ( (mb = new T_MB_MEDIA(ctx, mb_cnt)) != NULL)
  {
    TMediaFile *_mdf = TMediaFile::createMDF(fname);
    if (!_mdf) return NULL;
    
    if ( !(mb->Setup(_mdf, _astream, _vstream)) ) 
    {
      delete mb;  
      return NULL;
    }
    AppendMember(mb);
    SDL_Delay(100);     
  }
  return mb;
}  

int T_LST_MEDIA::SaveHeader(FILE *lst_file)
{
  if (dist_sum > 0.0)
  {   
    strcpy(lst_header.magic, MEDIA_BUF_MAGIC);
    lst_header.dist_sum = dist_sum;
    lst_header.mb_cnt   = mb_cnt;
    
    fwrite(&lst_header, sizeof(T_MEDIA_HDR), 1, lst_file);
    return 1;
  }
  return 0;
}

int T_LST_MEDIA::LoadHeader(FILE *lst_file)
{
  int         rc = 0;

  rc = fread(&lst_header, sizeof(T_MEDIA_HDR), 1, lst_file);
  if (rc) rc = ( 0 == strcmp(lst_header.magic, MEDIA_BUF_MAGIC) );

  if (!rc)
  {
    fprintf(stderr,"WARNING: this is no media-list file\n");
    media_id = 0;
    dist_sum = 0;
    mb_cnt   = 0;  

    return 0;
  }
  else
  {
    dist_sum = lst_header.dist_sum;
    return 1;
  }
}

void T_LST_MEDIA::CommitHeader(void)
{
  dist_sum = lst_header.dist_sum;
}

T_MEMBER* T_LST_MEDIA::CreateMember(int n)
{
  return (T_MEMBER *)new T_MB_MEDIA(ctx, n);
} 
