/*
 *  mediafile.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "lve.h"
#include "mediafile.h"
#include "decoder.h"

TMediaFile *TMediaFile::list[MAX_MEDIA_CNT] = {};
int        TMediaFile::count                = 0;

void TMediaFile::cleanup(void)
{
  id       = -1;
  refCount = 1;
  isOpen   = 0;
  videoW   = 0;
  videoH   = 0;
  frmSrv   = NULL;
  ovFmt    = 0;
  strcpy(fn,"");
}

TMediaFile::TMediaFile(char *_fn, int _id)
{
  //fprintf(stderr, "*** DEBUG: new MDF instance *** \n");
  cleanup();
  id = _id;
  strcpy(fn, _fn);  
}

TMediaFile::~TMediaFile()
{
  //fprintf(stderr, "*** DEBUG: remove MDF reference *** \n");
  lve_close_mdf(this);
}

TMediaFile *TMediaFile::createMDF(char *fname)
{
  int i;
  struct stat stat_buf;
  
  for (i=0; i<count; i++)
  {
    if ( !strcmp(list[i]->fn, fname) ) 
    {
      //fprintf(stderr, "*** DEBUG: new MDF reference (create) *** \n");
      list[i]->refCount++;
      return list[i];
    }
  }

  //-- check availability of this file -- 
  if (stat(fname, &stat_buf) == -1) return NULL;

  count++;
  
  if (count == MAX_MEDIA_CNT)
  {
    fprintf(stderr, "ERROR: max count of different media files reached !\n");
    return NULL;
  }
  
  list[i] = new TMediaFile(fname, i);
    
  return list[i];
}

TMediaFile *TMediaFile::dup(void)
{
  //fprintf(stderr, "*** DEBUG: new MDF reference (dup) *** \n");
  refCount++;
  return this;
}

void TMediaFile::releaseMDF(TMediaFile *mdf)
{
  int i = mdf->id;
  
  if (count)
  {
    mdf->refCount--;
    if (!mdf->refCount) 
    {
      //fprintf(stderr, "*** DEBUG: release MDF instance id (%d), cnt (%d/%d)*** \n", i, count, mdf->refCount);
      delete mdf;
      count--;
      
      if (count)
      {
        list[i]     = list[count];
        list[i]->id = i;
      }
      list[count] = NULL;
    }
  }  
}

void TMediaFile::resetList(void)
{
  int i = 0;
  while (!list[i]) 
  {
    delete list[i];
    list[i] = NULL;
    i++;
  }  
    
}

TMediaFile **TMediaFile::getList(void)
{
  return list;
}

int TMediaFile::getId() 
{ 
  return id; 
}

char *TMediaFile::getName() 
{ 
  return fn; 
}

void TMediaFile::showInfo(void)
{
  fprintf(stderr, "File (%s) has ID (%d), (%d) Refs\n", fn, id, refCount); 
}

/*
int main(void)
{
  int i = 0;
  TMediaFile *p[3];
  TMediaFile **lst;
  
  p[0] = TMediaFile::createMDF("filename 1");
  p[1] = TMediaFile::createMDF("filename 2");
  p[2] = TMediaFile::createMDF("filename 1");  

  lst = p[0]->getList();
  while (lst[i])
  {
    lst[i]->showInfo();
    i++;
  }
  
  return 0;
}
*/

// eof
