/*
 *  region.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <SDL/SDL.h>
#include <stdlib.h>
#include <string.h>
#define _REGION_ 1
#include "region.h"
#include "display.h"

char     *T_REGION::lib_path   = NULL;
T_REGION **T_REGION::rg_list   = NULL;
T_REGION *T_REGION::first_rg   = NULL;
T_REGION *T_REGION::prev_rg    = NULL;
T_LVE_CTX  *T_REGION::ctx      = NULL;
Uint8    *T_REGION::screen_dup = NULL;
int       T_REGION::n_region   = 0;

T_REGION *T_REGION::rg_down  = NULL;
T_REGION *T_REGION::rg_drag  = NULL;
T_REGION *T_REGION::rg_focus = NULL;
int	 T_REGION::rg_down_num  = 0;
int	 T_REGION::rg_drag_num  = 0;
char     T_REGION::rg_drag_move = 0;
int      T_REGION::rg_focus_num = 0;
int	 T_REGION::rg_new_page  = 0;
int	 T_REGION::rg_key_mod   = 0;

T_REGION::T_REGION(void)
{
  //-- store args --
  T_REGION::type  = RG_TY_INITIAL;
  T_REGION::id    = RG_ID_UNDEF;
  T_REGION::ksym  = RG_NOKEY;
  T_REGION::name  = "initial";
  T_REGION::cols  = 0;
  T_REGION::rows  = 0;
  T_REGION::n_rect= 0;

  list  = NULL;
  
  return;
}  


T_REGION::T_REGION(int type, int id, const char *name,
                   int ksym, int cols, int rows)
{
  int		j, r, n;
  FILE		*fd;
  char 		fname[1024];
  Uint8		*p;
  SDL_Rect      b_rect;
  SDL_Rect      *rect;
  
  //-- store args --
  T_REGION::type = type;
  T_REGION::id   = id;
  T_REGION::ksym = ksym;
  T_REGION::name = name;
  T_REGION::cols = cols;
  T_REGION::rows = rows;

  //-- append this to end of list --
  if (prev_rg) prev_rg->next_rg = this;
  prev_rg = this;
  next_rg = NULL;
  
  //-- store pointer for first element --
  if (!n_region) first_rg = this;
   
  //-- adjust counter --
  n_region++;
  n_rect = 1;

  list  = NULL;
  page0 = flag0 = 0;
  ppage = &page0;
  pflag = &flag0;

  sf[0]   = NULL;
  sf[1]   = NULL;
  fit_buf = NULL;  
  
  //====================== 
  //== setup region now == 
  //======================

  //== read coordinate definition(s) of region element ==
  strcpy(fname, lib_path);
  strcat(fname, name);
  strcat(fname, RG_EXT_RECT);
  if ((fd = fopen(fname, "rb")) != NULL)
  {
    //== create array of rects definitions ==  
    n_rect = cols * rows;
    rect   = new SDL_Rect[n_rect];
       
    //-- we have it ? -- 
    if (rect != NULL)
    {  
      int  want_bmp, n_read;
      int  screen_dup_w;
      int  w_dup, h_dup;
      int  x_dup, y_dup;

      //-- bind this rect-array to region element -- 
      dst_rect = rect;
      
      //-- read rect defintions of first element (!) --
      n_read = fread(rect, sizeof(SDL_Rect), 1, fd);

      //== region tables(!) needs extended definition ==
      if (n_read && (n_rect > 1))
      {
        //== read table endpoint coordinats (of last rect) ==
        if (fread(&b_rect, sizeof(SDL_Rect), 1, fd) == 1)
        {
            b_rect.w = b_rect.x - rect[0].x;
            b_rect.h = b_rect.y - rect[0].y;
 
            //-- calc brutto width and heights for one table element: --
            //-- b_rect.w = rect.w + <gap_x>; 
            //-- b_rect.h = rect.h + <gap_y>;  
            b_rect.w = rect[0].w + (int)( (b_rect.w - rect[0].w * cols + 0.5)/(cols-1.0) );
            b_rect.h = rect[0].h + (int)( (b_rect.h - rect[0].h * rows + 0.5)/(rows-1.0) );
            
            //== now calc coordinates of each table element starting at second ==
            n = 1;
            for (r = 0; r < rows; r++)
            {  
              for (j = 0; j < cols; j++)
              { 
                //-- first table element is already known --
                if (!j && !r) continue; 
                
                //-- calc coordinates by multiple of brutto width (height) --
                //-- in addition to the first-table element coordinates    --
                rect[n].x = rect[0].x + j * b_rect.w;
                rect[n].y = rect[0].y + r * b_rect.h;
                
                //-- same display width and height for each --
                rect[n].w = rect[0].w;
                rect[n].h = rect[0].h;
                
                //#define rr rect[n]
                //printf (" -> %d: (%d,%d) (%d x %d)", n, rr.x, rr.y, rr.w, rr.h); 
                n++;
              }    
            }  
        }
        else
        {
            fprintf(stderr, "Error: rg_list_init() - table defs not found in (%s)\n", name);
            
            //-- prevent unneeded allocation of region array --
            n_rect = 1;             
            cols   = 1;
            rows   = 1;
        }
      }
        
        
      //== for this region fillup corresponend rects in <dup_screen> ==
      screen_dup_w = ctx->screen->w>>1;
      for (j = 0; j < n_rect; j++)
      { 
        x_dup = rect[j].x>>1;
        y_dup = rect[j].y>>1;
        w_dup = rect[j].w>>1;
        h_dup = rect[j].h>>1;
        
        p  = screen_dup + (screen_dup_w * y_dup + x_dup);
        for (r = 0; r < h_dup; r++) 
        {
            memset(p, (Uint8)id, w_dup);
            p += screen_dup_w;
        }
      }
	
      //== read bitmap file(s) if wanted ==
      sf[0]    = NULL;
      sf[1]    = NULL;
      want_bmp = 1;
        
      switch (type)
      {
        case RG_TY_TEXT:
          strcpy(fname, lib_path);
          strcat(fname, name);
          strcat(fname, RG_EXT_BMP);
          
          //-- create surface to clear text --
          sf[0] = SDL_LoadBMP(fname);
          want_bmp = 0;
          break;

        case RG_TY_DRAG:
        case RG_TY_DROP:
          strcpy(fname, lib_path);
          strcat(fname, name);
          strcat(fname, RG_EXT_BMP);
       
          //-- create foreground and background surface --
          sf[0] = SDL_LoadBMP(fname);
          sf[1] = SDL_LoadBMP(fname);
          break;
            
        case RG_TY_SLIDER:    
          strcpy(fname, lib_path);
          strcat(fname, name);
          strcat(fname, "0");
          strcat(fname, RG_EXT_BMP);
          
          //-- create foreground surface --
          sf[0] = SDL_LoadBMP(fname);
          
          strcpy(fname, lib_path);
          strcat(fname, name);
          strcat(fname, "1");
          strcat(fname, RG_EXT_BMP);
          
          //-- create backgroud surface --
          sf[1] = SDL_LoadBMP(fname);
          break;
            
        default:
          want_bmp = 0;
          break;  
      }
        
      if (want_bmp)
      {  
        #define sf1 sf[1]
        if (sf1 != NULL)
        {
            //Uint8 bpp;
            //bpp = sf1->format->BytesPerPixel;
            //p   = (Uint8 *)sf1->pixels;
            //for (j = 0; j < (sf1->w * sf1->h * bpp); j++) p[j] = 128; // ~p[j];
            
            //-- save region of screen in bitmap (background) --
            SDL_BlitSurface(ctx->screen, rect, sf1, NULL);
        }
        else 
          fprintf(stderr, "Error: T_REGION::T_REGION(...) - reading file (%s)\n", fname);
      }
    }
    else
      fprintf(stderr, "Error: rg_list_init() - out of memory loading (%s)\n", fname);
       
    fclose(fd);  
  } 
  else
    fprintf(stderr, "Error: rg_list_init() - reading file %s\n", fname);

  //== table regions need further things ==    
  if (n_rect > 1)
  {
    //-- <fit_buf> is a special helper to find the right table element --
    //-- under the actual mouse position, similar <screen_dup>         --
    if ((fit_buf = new Uint8[ctx->screen->w + ctx->screen->h]) != NULL)
    {
      int r, ofs;
        
      memset(fit_buf, 255, ctx->screen->w + ctx->screen->h);
        
      for (j=0; j < cols; j++)
      {
        ofs = dst_rect[j].x;
        for (r = 0; r < dst_rect[0].w; r++)
          fit_buf[ofs+r] = j;
      }
        
      for (j=0; j < rows; j++)    
      {
        ofs = ctx->screen->w + dst_rect[j * cols].y;
        for (r = 0; r < dst_rect[0].h; r++)
          fit_buf[ofs + r] = j;   
      }
      
      ppage = new int[n_rect];
      pflag = new int[n_rect];
      for (j = 0; j < n_rect; j++)
      {
        ppage[j] = 0;
        pflag[j] = 0;
      }  
    }
    else
      fprintf(stderr, "Error: rg_list_init() - out of memory creating fit_buf in (%s)\n", name);
  }

}

void T_REGION::rg_setup(T_LVE_CTX *ctx, char *path)
{
  int w_dup = ctx->screen->w>>1; 
  int h_dup = ctx->screen->h>>1;

  T_REGION::ctx      = ctx;
  T_REGION::lib_path = path;
  
  //-- create (empty) region-screen 1/4 size of ctx->screen --
  if ((screen_dup = new Uint8[w_dup * h_dup]) == NULL)
    fprintf(stderr, "Error: T_REGION::rg_setup() - no memory for region screen\n");
  else
    memset(screen_dup, RG_ID_UNDEF, w_dup * h_dup);
}

void T_REGION::rg_sort(void)
{
  T_REGION *rg;
  int      i;
  
  //-- create list for <n_region> sorted region pointers --
  
  //-- ! to fix ! --
  rg_list = new T_REGION *[n_region+1];  
  if (rg_list)
  {
    rg_list[0] = NULL;
    rg = first_rg;
    do
    {
      rg_list[rg->id] = rg;
      
    } while ((rg = rg->next_rg) != NULL);  
  }
  else
  {
    rg_list = NULL;
    fprintf(stderr, "Error: rg_list_init() - no memory for sorted region list\n");
  }
}

void T_REGION::rg_cleanup(void)
{
  int i;

  if (type != RG_TY_INITIAL)
  {
    for (i=0; i<2; i++)
    {
      if (sf[i]) SDL_FreeSurface(sf[i]);
    }
    
    if (dst_rect) delete dst_rect;
  
    if ((rows > 1) && (cols > 1))
    {
      if (fit_buf) delete fit_buf;
    }
  }
  else
  {  
    if (screen_dup) delete screen_dup; 
    screen_dup = NULL;
  
    if (rg_list) delete rg_list;
    rg_list = NULL;
  }  
}

int T_REGION::rg_draw(int page, int num, T_MEMBER *mb)
{
  if (dst_rect && sf[page])
  {
    //-- blit foreground or background page --
    SDL_BlitSurface(sf[page], NULL, ctx->screen, &dst_rect[num]);
 
    //-- on background page - no further operations --
    if (page)
    { 
      ctx->refresh_bg = 1;
      return 1;
    }  
    
    //-- data list need special member handling -- 
    if (list)
    { 
      if (mb == NULL) mb = list->GetMember(num);
      
      if (mb != NULL)
      {  
        int need_thumb;
        
        //-- eventually mark rect --
        if (mb->HasMark())
        { 
          pflag[num] = RG_OFF;
          need_thumb = rg_invert(RG_ON, num, mb);
        }
        else
          need_thumb = 1;
          
        //--  blit thumbnail of this member --
        if (need_thumb) mb->BlitThumb(&dst_rect[num]);
      }   
    }
    //-- without list - only mark rect --
    else if (pflag[num]) 
    {
      pflag[num] = RG_OFF;
      rg_invert(RG_ON, num);
    }

    ctx->refresh_bg = 1;    
    return 1;
  }
  return 0;  
}                 

int T_REGION::rg_invert(int new_flag, int num, T_MEMBER* mb)
{
  if (new_flag == pflag[num]) return 0;
  pflag[num] = new_flag;
 
  if (dst_rect)
  {
    Uint8    *p; 
    int      bpp, w;
    int      c, r;
    SDL_Rect *rect;
    
    rect = &dst_rect[num];
    
    //-- get bytes per pixel --
    bpp = ctx->screen->format->BytesPerPixel;
    
    //-- get start offset in screen pixel-buffer --
    //-- and real wide (num of bytes)            --
    p = (Uint8 *)ctx->screen->pixels + rect->y * ctx->screen->pitch;
    p += (rect->x * bpp);
    w = rect->w * bpp;
    
    //-- invert rectangle inside the screen --
    for (r = 0; r < rect->h; r++) 
    {
      for (c = 0; c < w; c++)
      {
        p[c] = ~p[c];
      }
      p += ctx->screen->pitch; 
    }
    
    //-- we need refresh of display --
    ctx->refresh_bg = 1;
  }
  
  return 1;  // need drawing of thumb 
}                 

T_REGION& T_REGION::rg_get(int rg_id)
{
  return *(rg_list[rg_id]);
}

int T_REGION::rg_other_flags(int num)
{
  int i;
  
  for (i=0; i<RectCnt(); i++)
  {
    if (pflag[i] && (i != num))
      return 1;
  }
  return 0;    
}

static T_LIST   *last_lst = NULL;
static T_MEMBER *last_mb  = NULL;
void T_REGION::rg_single_focus(int num, int force)
{
  if (force || rg_new_page || !pflag[num] || rg_other_flags(num)) 
  {
    rg_new_page = 0;
    
    if (rg_focus) rg_focus->ClearAllMark();
    if ( rg_focus != this ) this->OnChangeFocus(num);

    if (last_lst)
    {
      last_lst->ClearFocusMember();
      last_lst = NULL;
      last_mb  = NULL;
    } 
    
    rg_focus     = this;
    rg_focus_num = num;
    
    //-- on list send signal to member --
    if ( list )
    { 
      last_mb  = list->OnSelectMember(num);
      last_lst = list;
    }  
    
    //-- now start action on focus --
    OnFocus(num);
    
    SetMark(num);  
  }
}
 
void T_REGION::OnMouseCommonUp(void)
{
  if (rg_down)
  {
    rg_down->ClearMark(rg_down_num);
    rg_down->OnCommonUp();
    rg_down = NULL;
  }
  display_set_cursor(0);
  rg_drag = NULL;
}
 
void T_REGION::OnFocus(int num)
{
  fprintf(stderr, "T_REGION::OnFocus() - focus on (%s, %d)\n", GetName(), num); 
} 
 
void T_REGION::OnMouseLeft(int num)
{
  OnMouseLeftDown(num);
  OnMouseLeftUp(num);
  OnMouseCommonUp();
}

void T_REGION::OnMouseRight(int num)
{
  OnMouseRightDown(num);
  OnMouseRightUp(num);
  OnMouseCommonUp();
}

int T_REGION::rg_event_mouse(int ev_type, SDL_MouseButtonEvent *mouse, int x, int y)
{
  int num = 0;
  int ret = RG_NOKEY;

  if (fit_buf)
  {
    num = fit_buf[x] + cols * fit_buf[ctx->screen->w + y];
  }       

  //-- mouse events allowed here ? --
  if (IsSensitive(num)) switch(mouse->button)
  {
    case SDL_BUTTON_LEFT:
      if (ev_type == SDL_MOUSEBUTTONDOWN) switch(rg_key_mod & ~KMOD_NUM)
      {
        case KMOD_LSHIFT:
        case KMOD_RSHIFT:
          OnShiftMouseLeftDown(num);
          break;
          
        case KMOD_LCTRL:
        case KMOD_RCTRL:
          OnCtrlMouseLeftDown(num);
          break;
        
        case KMOD_LALT:
        case KMOD_RALT:
          OnAltMouseLeftDown(num);
          break;
        
        default:
          ret = OnMouseLeftDown(num);
          break;
      }
      else switch (rg_key_mod)
      {
        case KMOD_LSHIFT:
        case KMOD_RSHIFT:
          OnShiftMouseLeftUp(num);
          break;
          
        case KMOD_LCTRL:
        case KMOD_RCTRL:
          OnCtrlMouseLeftUp(num);
          break;
        
        case KMOD_LALT:
        case KMOD_RALT:
          OnAltMouseLeftUp(num);
          break;
          
        default:
          ret = OnMouseLeftUp(num);
      }
      break;
      
    case SDL_BUTTON_RIGHT:
      if (ev_type == SDL_MOUSEBUTTONDOWN) switch (rg_key_mod)
      {
        case KMOD_LSHIFT:
        case KMOD_RSHIFT:
          ret = OnShiftMouseRightDown(num);
          break;
          
        case KMOD_LCTRL:
        case KMOD_RCTRL:
          ret = OnCtrlMouseRightDown(num);
          break;
        
        case KMOD_LALT:
        case KMOD_RALT:
          ret = OnAltMouseRightDown(num);
          break;

        default:
          ret = OnMouseRightDown(num);
      }
      else switch (rg_key_mod)
      {
        case KMOD_LSHIFT:
        case KMOD_RSHIFT:
          ret = OnShiftMouseRightUp(num);
          break;
          
        case KMOD_LCTRL:
        case KMOD_RCTRL:
          ret = OnCtrlMouseRightUp(num);
          break;
        
        case KMOD_LALT:
        case KMOD_RALT:
          ret = OnAltMouseRightUp(num);
          break;

        default:
          ret = OnMouseRightUp(num); 
        break;
      }   
      break;
      
    default:
      break;  
  }
  
  if (ev_type == SDL_MOUSEBUTTONUP) rg_get(id).OnMouseCommonUp();
    
  return ret;
}

int T_REGION::GetId(int x, int y)
{
  return screen_dup[(ctx->screen->w>>1) * (y>>1) + (x>>1)];
}

int T_REGION::rg_event_select(SDL_Event *event)
{
  static T_REGION *last_rg = NULL;
  
  int id, x, y;
  int ret         = RG_NOKEY;
  int is_key_down = 0; 
 
  switch(event->type)
  {
    case SDL_MOUSEBUTTONDOWN:
    case SDL_MOUSEBUTTONUP:
      
      //-- on mouse down: setup key modifier --     
      if (event->type == SDL_MOUSEBUTTONDOWN)
        rg_key_mod = SDL_GetModState();
      
      x = event->button.x;
      y = event->button.y;
      
      //-- we need the ID of mouse-clicked region --
      id = GetId(x,y);
      
      //-- handle only valid IDs --
      if (id != RG_ID_UNDEF)
      {
        last_rg = &rg_get(id);
        
        //-- now do region specific action --
        ret = last_rg->rg_event_mouse(event->type, &(event->button), x, y);
      }
      
      //-- on mouse up: reset key modifier -- 
      if (last_rg && event->type == SDL_MOUSEBUTTONUP) 
      {
        if (id == RG_ID_UNDEF) last_rg->OnMouseCommonUp();
        rg_key_mod = 0;
      }  
      break;
    
    case SDL_KEYDOWN:
    case SDL_KEYUP:
      ret = event->key.keysym.sym;
      
      //-- search regions for shortcut-keys --
      for (id=1; id<=n_region; id++)
      {
        if (rg_get(id).ksym == ret)
        {
          if (event->type == SDL_KEYDOWN)
          {
            rg_key_mod = SDL_GetModState(); 
            switch(rg_key_mod)
            {
              case KMOD_LSHIFT:
              case KMOD_RSHIFT:
                ret = rg_get(id).OnShiftMouseLeftDown(0);
                break;
          
              case KMOD_LCTRL:
              case KMOD_RCTRL:
                ret = rg_get(id).OnCtrlMouseLeftDown(0);
                break;
        
              case KMOD_LALT:
              case KMOD_RALT:
                ret = rg_get(id).OnAltMouseLeftDown(0);
                break;
                 
              default:
                ret = rg_get(id).OnMouseLeftDown(0);
            }
          }
          else
          {
            rg_key_mod = 0;
            rg_get(id).OnMouseCommonUp();
            ret = rg_get(id).OnMouseLeftUp(0);
          }
          break;
        }
      }
      break;
      
    default:
      ;  
  }
     
  return (ret);
}

int T_REGION::OnShiftMouseLeftDown(int num)
{
  if (rg_focus == this) SetGroupMark(num);
  
  return RG_NOKEY;
}

int T_REGION::OnCtrlMouseLeftDown(int num)
{
  if (rg_focus == this)
  {
    SetMark(num);
  }
  else
    rg_single_focus(num);
  
  return RG_NOKEY;
}

T_LIST *T_REGION::GetList(void) 
{  
  return list; 
}

T_LIST *T_REGION::GetListGlobal(void)
{
  if (rg_focus)
    return rg_focus->GetList();
  else
    return NULL; 
}

int T_REGION::GetPosGlobal(void)
{
  if (rg_focus) 
    return rg_focus->GetPos();
  else
    return 0;
}

void T_REGION::SetMark(int num)
{
  T_MEMBER* mb;
  
  if (list)
  {
    if ((mb = list->GetMember(num)) != NULL) 
      mb->SetMark(1);
  }
  rg_invert(RG_ON, num, mb);
}

void T_REGION::ClearMark(int num)
{
  T_MEMBER* mb;
  
  if (list)
  {
    if ((mb = list->GetMember(num)) != NULL) 
      mb->SetMark(0);
  }
  rg_invert(RG_OFF, num, mb);
}

void T_REGION::ClearAllMark(void)
{
  T_MEMBER* mb;
  int       i;
    
  if (list)
  {
    for (i=0; i < RectCnt(); i++) pflag[i] = 0;
    
    mb  = list->GetFirstMember();
    while (mb)
    {
      mb->SetMark(0);
      mb = list->GetNextMember(mb);
    }  
    DisplayUpdate(list->GetStartMember());
  }
  else
    for (i=0; i < RectCnt(); i++) rg_invert(RG_OFF, i);
}

void T_REGION::SetAllMark()
{
  T_MEMBER* mb;
  int       i;
  
  if (list)
  {
    for (i=0; i < RectCnt(); i++) pflag[i] = 1;
    
    mb  = list->GetFirstMember();
    while (mb)
    {
      mb->SetMark(1);
      mb = list->GetNextMember(mb);
    }
    DisplayUpdate(list->GetStartMember());  
  }
  else
    for (i=0; i < RectCnt(); i++) rg_invert(RG_ON, i);
}

void T_REGION::SetGroupMark(int num)
{
  T_MEMBER *mb, *mb1;
  int      i;
     
  if (list)
  {
    mb  = list->GetFocusMember();
    mb1 = list->GetMember(num);
    
    while ((mb = list->GetNextMember(mb)) != NULL)
    {
      mb->SetMark(1);
      if (mb == mb1) break;
    } 
    
    DisplayUpdate(list->GetStartMember());
  }
}  

void T_REGION::ClearGroupMark(int num)
{
  T_MEMBER *mb, *mb1;
  int      i;
    
  if (list)
  {
    mb  = list->GetFocusMember();
    mb1 = list->GetMember(num);
   
    while ((mb = list->GetNextMember(mb)) != NULL)
    {
      mb->SetMark(0);
      if (mb == mb1) break;
    }
    DisplayUpdate(list->GetStartMember()); 
  }
}

void T_REGION::NextPage(void)
{
  if (list) 
  {
    T_MEMBER* mb;
    
    if ( (mb = list->NextPage()) != NULL )
    {
      rg_new_page = 1;
      DisplayUpdate(mb);
    }  
  }
}
    
void T_REGION::PrevPage(void)
{
  if (list)
  {
    T_MEMBER* mb;
    
    if ( (mb = list->PrevPage()) != NULL)
    {
      rg_new_page = 1;
      DisplayUpdate(mb);
    }
  }
}

void T_REGION::MemberPage(void)
{
  if (list)
  { 
    T_MEMBER* mb;
  
    if ( (mb = list->MemberPage()) != NULL)
      DisplayUpdate(mb);
  }
}

void T_REGION::DisplayUpdate(T_MEMBER *mb)
{
  int i = 0;

  if (!list) 
  {
    fprintf(stderr, "ERROR: T_REGION::DisplayUpdate() - no list !\n");
    return;
  }  

  if (!mb)
  {
    mb = list->GetFocusMember();
    i  = rg_focus_num;
  }    
  
  while (mb && ( i < RectCnt()) )
  {
    rg_draw(0, i, mb);
    mb = list->GetNextMember(mb);    
    i++;
  }
  DisplayClearRest();
}

void T_REGION::DisplayClearRest(void)
{
  int i = list->MemberPageCnt();
 
  PrintPageNum();
  for (; i < RectCnt(); i++) rg_draw(1, i);
}

void T_REGION::SwitchOff(int num)
{ 
  rg_invert(RG_OFF, num);
}

void T_REGION::SwitchOn(int num)
{ 
  rg_invert(RG_ON, num);
}

//=====================
//== class T_RG_TEXT ==
//=====================
void T_RG_TEXT::init(void)
{
  text  = (char *)malloc(16+1);
  *text = '\0';
  len   = 16; 
}

void T_RG_TEXT::Print(char *_text, int clear)
{
  if (dst_rect)
  {
    if (clear) Clear();
    DT_DrawText(_text, ctx->screen, 0, dst_rect[0].x, dst_rect[0].y);
    if (text)
    { 
      int l = strlen(text) + strlen(_text);
      if (l > len) 
      {
        text = (char *)realloc(text, l+1);
        len  = l+1;
      }
      strcat(text, _text);
    }
  }
}

void T_RG_TEXT::Clear(void)
{ 
  if (text) strcpy(text,""); 
  if (sf[0] && dst_rect)
    SDL_BlitSurface(sf[0], NULL, ctx->screen, &dst_rect[0]);
}

char * T_RG_TEXT::Get(void)
{
  return text;
}

//=======================
//== class T_RG_TOGGLE ==
//=======================
int T_RG_TOGGLE::OnMouseLeftDown(int num)
{ 
      T_REGION::OnMouseLeftDown(num);
      rg_invert(!pflag[num], num);
      
      return RG_NOKEY;
}

int T_RG_TOGGLE::OnCtrlMouseLeftDown(int num)
{
  return OnMouseLeftDown(num);
}  

//=====================
//== class T_RG_DOWN ==
//=====================
int T_RG_DOWN::OnMouseLeftDown(int num)
{
      T_REGION::OnMouseLeftDown(num);
      rg_invert(RG_ON, num);
      
      rg_down     = this;
      rg_down_num = num;
      
      return RG_NOKEY;
}

int T_RG_DOWN::OnShiftMouseLeftDown(int num)
{
  return OnMouseLeftDown(num);
}  

int T_RG_DOWN::OnCtrlMouseLeftDown(int num)
{
  return OnMouseLeftDown(num);
}  

//=====================
//== class T_RG_DRAG ==
//=====================
int T_RG_DRAG::OnMouseLeftDown(int num)
{
      T_REGION::OnMouseLeftDown(num);
         
      display_set_cursor(1);
      if (!pflag[num]) rg_single_focus(num);
      rg_drag      = this;
      rg_drag_num  = num;
      rg_drag_move = 0;
      
      return RG_NOKEY;
}
    
int T_RG_DRAG::OnMouseLeftUp(int num)
{
  if (rg_drag == this) rg_single_focus(rg_drag_num);
  rg_drag = NULL;
  return RG_NOKEY;
}

//=====================
//== class T_RG_DROP ==
//=====================
void T_RG_DROP::OnDrop(int num, char mv_flag)
{
  int i;
  
  printf("drag (%s,)", rg_drag->GetName());
  for (i=0; i<rg_drag->RectCnt(); i++)
    if (rg_drag->IsSelected(i)) printf(" %d ", i);
    
  printf(") -> drop (%s, %d)\n", GetName(), num);
}
  
int T_RG_DROP::OnMouseLeftDown(int num)
{
      
  if ( IsSelectable(num) ) 
  {
    T_REGION::OnMouseLeftDown(num);
    if ( !pflag[num] ) rg_single_focus(num);
  }
  return RG_NOKEY;
}
    
int T_RG_DROP::OnMouseLeftUp(int num)
{
  int i;
      
  if (rg_drag)
  { 
    if ( (rg_drag != this) || (rg_drag_num != num) ) 
    { 
      OnDrop(num, rg_drag_move);
      
      //-- set single focus Mark, if this region holds focus --      
      rg_drag->ClearAllMark();
      if (rg_focus == this) SetMark(rg_focus_num);
  
      return RG_NOKEY;
    }
    rg_single_focus(num,1);
  }
  return RG_NOKEY;
}

//==========================
//== class T_RG_DRAGNDROP ==
//==========================
int T_RG_DRAGNDROP::OnMouseLeftDown(int num)
{
  if (IsSelectable(num))
  {
    T_RG_DROP::OnMouseLeftDown(num);
        
    display_set_cursor(1);      
    rg_drag      = this;
    rg_drag_num  = num;
    rg_drag_move = 0;
  }
  return RG_NOKEY;
}

int T_RG_DRAGNDROP::OnMouseRightDown(int num)
{
  OnMouseLeftDown(num);
  rg_drag_move = 1;
      
  return RG_NOKEY;
}

int T_RG_DRAGNDROP::OnMouseRightUp(int num)
{
  OnMouseLeftUp(num);
  return RG_NOKEY;
}


//=======================
//== class T_RG_SLIDER ==
//=======================
int T_RG_SLIDER::OnMouseLeftDown(int num)
{
  T_REGION::OnMouseLeftDown(num);
      
  rg_down     = this;
  rg_down_num = num;
      
  return RG_NOKEY;
}

void T_RG_SLIDER::Setup(void)
{
      can_draw = (dst_rect != NULL && sf[0] != NULL && sf[1] != NULL)? 1 : 0;
      if (can_draw)
      {
        sl_w_half = (sf[0]->w)>>1;
        
        rect.x = 0;
        rect.y = dst_rect[0].y;
        rect.w = sf[0]->w; 
        rect.h = sf[0]->h;
        
        sl_min_x = dst_rect[0].x + sl_w_half;       
        sl_max_x = dst_rect[0].x + dst_rect[0].w - sl_w_half;
        sl_w     = sl_max_x - sl_min_x; 
      } 
}
    
int T_RG_SLIDER::rg_draw(int page, int num, T_MEMBER *mb)
{
  if (can_draw)
  {
    //-- draw background first --
    SDL_BlitSurface(sf[1], NULL, ctx->screen, &dst_rect[0]);

    //-- now draw button on right x posiion --
    if (page)
    {  
      rect.x = num;      
      SDL_BlitSurface(sf[0], NULL, ctx->screen, &rect);
    }    
    rect.x = rect_x_sav;
    
    ctx->refresh_bg = 1;
    
    return 1;
  }
  
  return 0;  
}
    
//============================
//== some Helper C-Routines ==
//============================

static int is_locked = 0;
void rg_event_lock(void)
{
  is_locked = 1;
}

void rg_event_unlock(void)
{
  is_locked = 0;
}
   
int rg_event_islocked(void)
{
  return is_locked;
}    


// eof
