/*
 *  region.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifndef _REGION_H
#define _REGION_H 1

#include <SDL/SDL.h>
#include "rglist.h"
#include "DT_drawtext.h"

#define RG_EXT_BMP	".bmp"
#define RG_EXT_RECT	".rect"
#define RG_OFF		0
#define RG_ON		1
#define RG_NOKEY	-1

#define RG_TY_INITIAL	0
#define RG_TY_CHECK	1
#define RG_TY_RADIO	2
#define RG_TY_DOWN	3
#define RG_TY_TOGGLE	4
#define RG_TY_DRAG	5
#define RG_TY_DROP	6
#define RG_TY_TEXT	7
#define RG_TY_OV	8
#define RG_TY_SLIDER	9

#define RG_ID_UNDEF     0

#define RG_DIR_FORWARD  1
#define RG_DIR_BACK     0


//====================

extern void rg_event_lock(void);
extern void rg_event_unlock(void);
extern int  rg_event_islocked(void);

//====================

class T_REGION 
{
  private:
    static char*    lib_path;
    static T_REGION **rg_list;
    static T_REGION *first_rg;
    static T_REGION *prev_rg;

    T_REGION        *next_rg;

  protected:
    static T_LVE_CTX *ctx;
    static Uint8     *screen_dup;
    static int       n_region;
    static T_REGION  *rg_down;
    static T_REGION  *rg_drag;
    static T_REGION  *rg_focus;
    static int	     rg_down_num;
    static int       rg_drag_num;
    static char      rg_drag_move;
    static int       rg_focus_num;
    static int       rg_new_page;
    static int       rg_key_mod;
    
  protected:
    int type;
    int id;
    const char *name;
    int ksym;
    int cols;
    int rows;
    int n_rect;
    int flag0;
    int page0;
    int *pflag;
    int *ppage;
    SDL_Rect    *dst_rect;
    SDL_Surface *sf[2];
    Uint8       *fit_buf;
    T_LIST      *list;
    
  private: 
    int  rg_event_mouse(int ev_type, SDL_MouseButtonEvent *mouse, int x, int y);
    void rg_cleanup(void);
    int  rg_other_flags(int num);
   
  private: 
    static int GetId(int x, int y);
    
  public:
    T_REGION::T_REGION(void);
    
    T_REGION::T_REGION(int type, 
                       int id, 
                       const char *name,
                       int ksym, 
                       int cols=1, 
                       int rows=1);
  
    T_REGION::~T_REGION(void) 
    {
      rg_cleanup();
    };

    static void      rg_setup(T_LVE_CTX *ctx, char *path);
    static void      rg_sort(void);
    static int       rg_event_select(SDL_Event *event);
    static T_REGION& rg_get(int rg_id);
    static T_LIST    *GetListGlobal(void);      
    static int 	     GetPosGlobal(void);    

    const char *GetName(void) { return name; };    
    int         RectCnt(void) { return n_rect; };
    int         IsSelected(int num) { return pflag[num]; };
    int		GetRectX(int num) { return dst_rect[num].x; };
    int		GetRectY(int num) { return dst_rect[num].y; };
    void        SetList(T_LIST *lst) { list = lst; };
    void        rg_single_focus(int num, int force = 0);
    
    virtual int  rg_invert(int new_flag, int num, T_MEMBER* mb = NULL);
    virtual int  rg_draw(int page, int num, T_MEMBER* mb = NULL);
    
    virtual void ClearAllMark(void);
    virtual void SetAllMark(void);
    virtual void ClearMark(int num);
    virtual void SetMark(int num);
    virtual void ClearGroupMark(int num);
    virtual void SetGroupMark(int num);
        
    virtual void OnFocus(int num);
    virtual void OnMouseLeft(int num);
    virtual void OnMouseRight(int num);
    
    virtual void OnMouseCommonUp(void);
    
    virtual void OnChangeFocus(int num) {};
    virtual void OnCommonUp(void) {};
    
    virtual int OnMouseLeftDown(int num) {return RG_NOKEY; }; 
    virtual int OnMouseLeftUp(int num) {return RG_NOKEY; };
    virtual int OnMouseRightDown(int num) {return RG_NOKEY; };
    virtual int OnMouseRightUp(int num){return RG_NOKEY; };

    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnShiftMouseLeftUp(int num) {return RG_NOKEY; };
    virtual int OnShiftMouseRightDown(int num) {return RG_NOKEY; };
    virtual int OnShiftMouseRightUp(int num){return RG_NOKEY; };

    virtual int OnCtrlMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftUp(int num) {return RG_NOKEY; };
    virtual int OnCtrlMouseRightDown(int num) {return RG_NOKEY; };
    virtual int OnCtrlMouseRightUp(int num){return RG_NOKEY; };

    virtual int OnAltMouseLeftDown(int num) {return RG_NOKEY; };
    virtual int OnAltMouseLeftUp(int num) {return RG_NOKEY; };
    virtual int OnAltMouseRightDown(int num) {return RG_NOKEY; };
    virtual int OnAltMouseRightUp(int num){return RG_NOKEY; };

    virtual T_LIST *GetList(void);
    virtual int    GetPos(void) { return 0; };
    virtual int    IsSensitive(int num) { return 1; };
    virtual int    IsSelectable(int num) { return 1; };

    virtual void NextPage(void);
    virtual void PrevPage(void);
    virtual void MemberPage(void);    

    virtual void PrintPageNum(void) {};    
    virtual void DisplayUpdate(T_MEMBER *mb=NULL);
    virtual void DisplayClearRest(void);

    virtual void SwitchOff(int num);
    virtual void SwitchOn(int num);
};

#define REGION(_region_id) (T_REGION::rg_get(_region_id))

//============================================================================

class T_RG_TEXT : public T_REGION
{
  private:
    char *text;
    int  len;
     
  private:
    void init(void);  
    
  public:
    T_RG_TEXT::T_RG_TEXT(int id, const char *name) 
                        : T_REGION(RG_TY_TEXT, id, name, RG_NOKEY)
    {
      init();
    };
    
    T_RG_TEXT::T_RG_TEXT(int id, const char *name, int ksym, int cols, int rows) 
                        : T_REGION(RG_TY_TEXT, id, name, ksym, cols, rows)
    {
      init();
    };

    ~T_RG_TEXT(void)
    {
      if (text) delete text;
      text = NULL;
    };

    void Print(char* text, int clear = 1);
    void Clear(void);
    char *Get(void);
};

#define RG_TEXT(_region_id) ((T_RG_TEXT &)REGION(_region_id))

//============================================================================

class T_RG_TOGGLE : public T_REGION
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num);
    
    T_RG_TOGGLE::T_RG_TOGGLE(int id, const char *name, int ksym) 
                            : T_REGION(RG_TY_TOGGLE, id, name, ksym)
    {
    };
    
    T_RG_TOGGLE::T_RG_TOGGLE(int id, const char *name, int ksym, int cols, int rows) 
                            : T_REGION(RG_TY_TOGGLE, id, name, ksym, cols, rows)
    {
    };
};

class T_RG_DOWN : public T_REGION
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnShiftMouseLeftDown(int num);
    virtual int OnCtrlMouseLeftDown(int num);
    
    T_RG_DOWN::T_RG_DOWN(int id, const char *name, int ksym) 
                        : T_REGION(RG_TY_DOWN, id, name, ksym)
    {
    };
    T_RG_DOWN::T_RG_DOWN(int id, const char *name, int ksym, int cols, int rows) 
                        : T_REGION(RG_TY_DOWN, id, name, ksym, cols, rows)
    {
    };
};

class T_RG_DRAG : public T_REGION
{
  public:
    virtual int OnMouseLeftDown(int num);
    virtual int OnMouseLeftUp(int num);
    
    T_RG_DRAG::T_RG_DRAG(int id, const char *name, int ksym) 
                        : T_REGION(RG_TY_DRAG, id, name, ksym)
    {
    };
    
    T_RG_DRAG::T_RG_DRAG(int id, const char *name, int ksym, int cols, int rows) 
                        : T_REGION(RG_TY_DRAG, id, name, ksym, cols, rows)
    {
    };
};

class T_RG_DROP : public T_REGION
{
  public:
    virtual void OnDrop(int num, char mv_flag);
    virtual int  OnMouseLeftDown(int num);
    virtual int  OnMouseLeftUp(int num);

    T_RG_DROP::T_RG_DROP(int id, const char *name, int ksym, int cols, int rows) 
                        : T_REGION(RG_TY_DROP, id, name, ksym, cols, rows)
    {
    };
};

class T_RG_DRAGNDROP : public T_RG_DROP
{
  public:
    virtual int  OnMouseLeftDown(int num);
    virtual int  OnMouseRightDown(int num);
    virtual int  OnMouseRightUp(int num);


    T_RG_DRAGNDROP::T_RG_DRAGNDROP(int id, const char *name, 
                                   int ksym, int cols, int rows) 
      : T_RG_DROP(id, name, ksym, cols, rows)
    {
    };
};

class T_RG_OV : public T_REGION
{
  public:
    T_RG_OV::T_RG_OV(int id, const char *name) 
                        : T_REGION(RG_TY_OV, id, name, RG_NOKEY)
    {
    };
};

class T_RG_SLIDER : public T_REGION
{
  private:
    int      can_draw;
    int      rect_x_sav;
    
  protected:
    int      sl_min_x;
    int      sl_max_x;
    int      sl_w;
    int      sl_w_half;
    int      sl_hidden;
    
    SDL_Rect rect;
  
  protected:
    void Setup(void);
      
  public:
    T_RG_SLIDER::T_RG_SLIDER(int id, const char *name) 
                            : T_REGION(RG_TY_SLIDER, id, name, RG_NOKEY) 
    { 
      sl_hidden = 0;
      Setup(); 
    };

    virtual int  OnMouseLeftDown(int num);
    
    virtual int  rg_invert(int new_flag, int num, T_MEMBER* mb = NULL) 
    { 
      return 0; 
    };
    
    virtual int  rg_draw(int page, int num, T_MEMBER *mb = NULL);
};

#endif // _REGION_H 
