/*
 *  rglist.c
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include "rglist.h"
#include "region.h"

T_MEMBER::T_MEMBER(T_LVE_CTX *ctx, int num)
{
  T_MEMBER::ctx = ctx;
  mb_num        = num;
  has_mark      = 0;
}

T_MEMBER *T_MEMBER::OnSelect(void) 
{
  printf("member (%d) selected", mb_num);
  return this;
}

void T_MEMBER::Save(FILE *file)
{
  printf("saving member (%d)\n", mb_num);
  return;
}

int T_MEMBER::Load(FILE *file)
{
  printf("loading member (%d)\n", mb_num);
  return 1; // 1 = eof
}

//============================================================================

T_LIST::T_LIST(void) 
{
      ctx          = NULL;
      page_w       = 0;
      page_num     = 1;      
            
      mb_first     = NULL;
      mb_last      = NULL;
      mb_focus     = NULL;
      mb_start     = NULL;
      mb_focus_num = 0;
      mb_start_num = 0;
      mb_cnt       = 0;    

      mb_save_needed     = 0;
      mb_man_save_needed = 0;
}
    
T_LIST::~T_LIST(void)
{
  Reset();
}

void T_LIST::Reset(void)
{
  T_MEMBER *mb = mb_first;
  T_MEMBER *mb_sav;
     
  while (mb)
  {
    mb_sav = mb->mb_next;
    delete mb;
    mb = mb_sav;
  }
  
  page_num = 1;
  
  mb_first = NULL;
  mb_last  = NULL;
  mb_focus = NULL;
  mb_start = NULL;
  mb_start_num = 0;
  mb_focus_num = 0;
  mb_cnt       = 0;
  
  mb_save_needed     = 0;
  mb_man_save_needed = 0;
}

    
void T_LIST::Setup(T_LVE_CTX *ctx, int rect_cnt)
{
  T_LIST::ctx = ctx;
  page_w      = rect_cnt;
}
    
    
T_MEMBER *T_LIST::GetMember(int num)
{
  T_MEMBER *mb;
  int      r_num = mb_start_num + num;
      
  mb = mb_start;
  while (mb)
  {
    if (mb->mb_num == r_num)
      return mb;
        
    mb = mb->mb_next;  
  }
  return NULL;
} 

void T_LIST::RemoveMember(int num)
{
  RemoveMember(GetMember(num));
}

void T_LIST::RemoveMember(T_MEMBER *mb_rem, int del)
{
  T_MEMBER *mb_sav;

  if (!mb_rem) return;
  mb_sav = mb_rem->mb_next;  

  //-- important: adjust page --
  while (mb_rem->mb_num <= mb_start_num) 
  {
    if (mb_start_num == 0) break; 
    PrevPage();
  }
    
  if (mb_rem == mb_last) 
    mb_last = mb_rem->mb_prev;
  else
    mb_rem->mb_next->mb_prev = mb_rem->mb_prev;
    
  if (mb_rem == mb_first) 
    mb_first = mb_rem->mb_next;
  else 
    mb_rem->mb_prev->mb_next = mb_rem->mb_next;

  if (mb_rem == mb_start) 
    mb_start = mb_rem->mb_next;
    
  if (mb_rem == mb_focus) 
  { 
    mb_focus = mb_rem->mb_prev;
    if (!mb_focus)
      mb_focus_num = 0;
    else if (mb_focus_num > 0) 
      mb_focus_num--;
  } 
     
  while (mb_sav)
  {
    mb_sav->mb_num--;
    mb_sav = mb_sav->mb_next;
  }
  
  if (del) delete mb_rem;
  mb_cnt--;

  mb_save_needed = 1;
}

T_MEMBER *T_LIST::InsertMember(T_MEMBER *mb_new, T_MEMBER *mb)
{
  T_MEMBER *mb_sav = mb;
  int       num    = 0;
  
  if (!mb_new) return NULL; 
      
  if (mb == mb_first) mb_first = mb_new; 
  if (mb == mb_start) mb_start = mb_new;
  if (mb == mb_focus) mb_focus = mb_new;
  
  if (mb->mb_prev) mb->mb_prev->mb_next = mb_new;
  mb_new->mb_prev = mb->mb_prev;
  mb->mb_prev     = mb_new;
  mb_new->mb_next = mb;
  
  if (mb_sav) num = mb_sav->mb_num;
  mb_new->mb_num = num;
  
  while (mb_sav)
  {
    num++;
    mb_sav->mb_num = num;
    mb_sav = mb_sav->mb_next;
  }
      
  mb_cnt++;

  mb_save_needed = 1;  
  return mb_new;
}
    
T_MEMBER *T_LIST::AppendMember(T_MEMBER *mb_new)
{
  if (!mb_new) return NULL;
  
  mb_cnt++;      
  
  if (!mb_first)
  { 
    mb_first     = mb_new; 
    mb_start     = mb_new;
    mb_start_num = 0;
    
    mb_focus     = mb_new;
    mb_focus_num = 0;
  } 
        
  if (mb_last) 
    mb_last->mb_next = mb_new;
        
  mb_new->mb_prev = mb_last;
  mb_new->mb_next = NULL;
  mb_last         = mb_new;
      
      
  mb_save_needed = 1;    
  return (mb_new);
}

void T_LIST::UpdateMember(T_MEMBER *mb, int all)
{
  return;
}
    
T_MEMBER *T_LIST::OnSelectMember(int num)
{
  T_MEMBER *mb;
      
  if ((mb = GetMember(num)) != NULL)
  {
    if ((mb_focus = mb->OnSelect()) != NULL)
      mb_focus_num = num + mb_start_num;
    return mb_focus;        
  }
  return NULL;
}

T_MEMBER *T_LIST::NextPage(void)
{
  if (mb_start_num + page_w < MemberCnt())
  {
    int i;
    
    page_num++;
       
    mb_start_num += page_w;
    for (i=0; i<page_w; i++) mb_start = mb_start->mb_next;
    return mb_start;
  }
  return NULL;
}

T_MEMBER *T_LIST::PrevPage(void)
{
  if (mb_start_num - page_w >= 0)
  {
    int i;
   
    page_num--; if (page_num<1) page_num = 1;
    
    mb_start_num -= page_w;
    for (i=0; i<page_w; i++) mb_start = mb_start->mb_prev;
    return mb_start;
  }
  return NULL;
}

T_MEMBER *T_LIST::FirstPage(void)
{
  if (mb_start_num != 0)
  {
    page_num     = 1;
    
    mb_start_num = 0;
    mb_start     = mb_first;
    return mb_start;
  }
  return NULL;
}

T_MEMBER *T_LIST::LastPage(void)
{
  int num;
  int i;
    
  page_num = (int)((MemberCnt()-1) / page_w);
  num      = page_w * page_num;
  page_num ++;
    
  if (mb_start_num < num)
  {  
    mb_start = mb_last;
    for (i = MemberCnt(); i>num; i++)
      mb_start = mb_start->mb_prev;

    mb_start_num = num;
    return mb_start;
  }
  
  return NULL;
}

//-- gets Display-Number of a member, returns -1, if member not in page --
int T_LIST::MemberDisplayNum(T_MEMBER *mb)
{
  int n = -1;
  
  if (mb)
  {
    n = mb->mb_num - mb_start_num; 
    if (n >= page_w) n = -1;
  }   
  return n;
}

//-- chooses page containing focus-member, returns start-member --
T_MEMBER *T_LIST::MemberPage(T_MEMBER *chk_mb)
{
  T_MEMBER *mb = NULL;
  int      find_num, dir;
  
  if (!chk_mb)
    find_num = mb_focus_num;
  else
    find_num = chk_mb->GetNum();
      
  dir = find_num - mb_start_num;

  if (!dir) return NULL;
  
  if (dir > 0)
  {
    while (mb_start_num + page_w <= find_num)  mb = NextPage(); 
  }
  else   
  {
    while (mb_start_num > find_num) mb = PrevPage(); 
  }
  return mb;
}

void T_LIST::RequestSave(void)
{
  mb_save_needed = 1;
}

void T_LIST::SaveMembers(char *fname, int automatic)
{
  FILE     *file = NULL;
  T_MEMBER *mb   = NULL;
  
  if (!mb_cnt) return;
  
  if (automatic)
  {
    //if (mb_save_needed) mb_man_save_needed = 1;
    if (!mb_save_needed) return;
    mb_save_needed = 0;
  }
  /*
  else
  {
    if (!mb_man_save_needed) return;
    mb_man_save_needed = 0;
  }   
  */
  
  if ( (file = fopen(fname,"wb")) != NULL)
  {
    if (!SaveHeader(file))
    {
      fclose(file);
      return;
    }  
  
    mb = GetFirstMember();
    while(mb)
    {
      mb->Save(file);
      mb = mb->GetNext();  
    }
    fclose (file);      
  }
  else
    fprintf(stderr,"ERROR: T_LIST::SaveMembers - opening (%s) for writing\n", fname);
}

T_MEMBER *T_LIST::CreateMember(int n)
{
  return new T_MEMBER(ctx, n);
}

int T_LIST::LoadMembers(char *fname)
{
  FILE     *file = NULL;
  T_MEMBER *mb   = NULL;
  int      eof   = 0;
  int      rc    = 0;
  int      n;

  mb_save_needed = 0;
  
  if ( (file = fopen(fname, "rb")) != NULL ) 
  {
    if (!LoadHeader(file))
    {
      fprintf(stderr,"ERROR: T_LIST::LoadMembers - doesn't read media-list (%s)\n", fname);
      fclose(file);
      return 0;
    }
    
    //-- clear old list and commit readed header data --
    Reset();
    CommitHeader();
     
    //-- ... and load members of new list -- 
    n=0;     
    do
    {
      if ( (mb = CreateMember(n)) != NULL) 
      {
        n++;
        if ( (eof = mb->Load(file)) == 0)
        {
          T_LIST::AppendMember(mb);
          rc = 1;
        }
        else
          delete mb;
          
        mb_save_needed = 0;  
      } 
      else
      {
        fprintf(stderr, "ERROR: T_LIST::LoadMembers() - out of memory reading member out of (%s)\n", fname);
        break;
      }    
    } while (!eof);
    fclose(file);
  }
  else
  {
    //-- only clear old --
    Reset();
    fprintf(stderr,"INFO: T_LIST::LoadMembers - using (%s) as new media-list\n", fname);
  }  

  return rc;
}
