/*
 *  rglist.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifndef _RGLIST_H
#define _RGLIST_H 1

#include <stdio.h>
#include <string.h>
#include "lve_ctx.h"

class T_LIST;

class T_MEMBER
{
  friend class T_LIST;
  
  protected:
    T_LVE_CTX *ctx;  
    T_MEMBER  *mb_next;
    T_MEMBER  *mb_prev;

    int       has_mark;
    int       mb_num;
        
  public: 
  
    T_MEMBER::T_MEMBER(T_LVE_CTX *ctx, int num);
    
    int       GetNum(void) { return mb_num; };
    int	      HasMark(void) { return has_mark; };
    void      SetMark(int flag) { has_mark = flag; };
    T_MEMBER *GetNext() { return mb_next; };
    T_MEMBER *GetPrev() { return mb_prev; };

    virtual void     BlitThumb(SDL_Rect *dst_rect) {};
    virtual void     GenThumb(void) {};
    virtual T_MEMBER *OnSelect(void);
    virtual void     ShowInfo(void) {};

  protected:
    virtual void     Save(FILE *lst_file);
    virtual int      Load(FILE *lst_file);
};

class T_LIST
{
  protected:
    T_LVE_CTX  *ctx;
      
    T_MEMBER *mb_first;
    T_MEMBER *mb_last;
    T_MEMBER *mb_focus;
    T_MEMBER *mb_start;
    int       mb_start_num;
    int       mb_focus_num;
    int       mb_cnt;
    
    int       page_w;
    int       page_num;
    int       mb_save_needed;
    int       mb_man_save_needed;
    
  public:
  
    T_LIST::T_LIST(void);
    
    T_LIST::~T_LIST(void);
    
    int       GetPageNum() { return page_num; };
    int       GetNPages() { return ((int)((mb_cnt-1)/page_w) + 1); };
    int       MemberCnt(void) { return mb_cnt; };
    int       MemberPageCnt(void) { return (mb_cnt - mb_start_num); };
    int       MemberDisplayNum(T_MEMBER *mb);
    
    T_MEMBER *GetMember(int num);
    T_MEMBER *GetNextMember(T_MEMBER *mb) { return mb->mb_next; };
    T_MEMBER *GetPrevMember(T_MEMBER *mb) { return mb->mb_prev; };
    T_MEMBER *GetFirstMember() { return mb_first; };
    T_MEMBER *GetLastMember() { return mb_last; };
    T_MEMBER *GetStartMember() { return mb_start; };
    T_MEMBER *GetFocusMember() { return mb_focus; };
    T_MEMBER *NextPage(void);
    T_MEMBER *PrevPage(void);
    T_MEMBER *FirstPage(void);
    T_MEMBER *LastPage(void);
    T_MEMBER *MemberPage(T_MEMBER *chk_mb = NULL);
    void     ClearFocusMember() { mb_focus = NULL; };
    void     SaveMembers(char *fname, int automatic = 0);
    int      LoadMembers(char *fname);
    void     RequestSave(void);
    
    virtual void      Setup(T_LVE_CTX *ctx, int rect_cnt);
    virtual T_MEMBER *OnSelectMember(int num);
    
    virtual T_MEMBER *InsertMember(T_MEMBER *mb_new, T_MEMBER *mb);
    virtual T_MEMBER *AppendMember(T_MEMBER *mb_new);
    virtual void      RemoveMember(int num);
    virtual void      RemoveMember(T_MEMBER *mb_rem, int del = 1);
    virtual void      UpdateMember(T_MEMBER *mb, int all = 0);
    virtual void      Reset(void);

  protected:
    virtual int       SaveHeader(FILE *file) { return 0; };
    virtual int       LoadHeader(FILE *file) { return 0; };
    virtual void      CommitHeader(void) {};
    virtual T_MEMBER *CreateMember(int n);
};

#endif // _RGLIST_H
