/*
 *  scenelst.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifndef _SCENELST_H
#define _SCENELST_H 1

#include <string.h>
#include "lve.h"
#include "rglist.h"
#include "display.h"
#include "media_buf.h"

class T_LST_SCENE;
class T_LST_MEDIA;
class T_LST_EDIT;

class T_MB_EDIT;
class T_MB_MEDIA;

class T_MB_SCENE : public T_MEMBER
{
  friend class T_LST_SCENE;
  friend class T_LST_MEDIA;
  friend class T_LST_EDIT;
  friend class T_MB_EDIT;
  friend class T_MB_MEDIA;

  protected:
    static int last_mdf_id;  

  protected:
    TMediaFile *mdf;
    char       astream;
    char       vstream;
    int        start;
    int        end;
    int        start_real;
    int        end_real;
    int        dist;
    int        avs_dp;
    
    Uint8      bmp_thumb[BMP_TH_LEN];
  
  public:
    T_MB_SCENE(T_LVE_CTX *ctx, int num);
    ~T_MB_SCENE();
    
    int        Setup(TMediaFile *_mdf, int _astream, int _vstream); 
    void       SetStartFrame(int p);
    void       SetEndFrame(int p);
    int        GetStartFrame();
    int        GetEndFrame();
    int        FramesCnt();
    int        GetRealFrame(void);
    int	       GetMDFId();
    char       *GetMDFName();
    int        GetOutputSize();

    virtual T_MB_SCENE *Dup(void);
    virtual void       BlitThumb(SDL_Rect *dst_rect);
    virtual void       GenThumb(void);
    virtual T_MEMBER   *OnSelect(void); 
 
    virtual void ShowInfo(T_LST_SCENE *lst);   
};

class T_LST_SCENE : public T_LIST
{
  protected:
    int 	 dist_sum;
    int          media_id;  
    T_MEDIA_HDR  lst_header;
    
  public:
    T_LST_SCENE::T_LST_SCENE() : T_LIST()
    {
      dist_sum = 0;
      media_id = 0;
    };
    
    void  resetLastMDFId(void);
    int   FramesCnt(void) { return dist_sum; };
    off_t GetOutputSize(void);
  
    virtual T_MEMBER *InsertMember(T_MEMBER *mb_new, T_MEMBER *mb);
    virtual T_MEMBER *AppendMember(T_MEMBER *mb_new);
    virtual T_MEMBER *OnSelectMember(int num);
    virtual void     UpdateMember(T_MEMBER *mb, int all = 0);
};

#endif // _SCENELST_H
