/*
 *  yuv2rgb16.h
 *
 *  Copyright (C) Gerhard Monzel - October 2003
 *
 *  This file is part of lve, a linux mpeg video editor
 *      
 *  lve tools are free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  lve is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#define B f[0]
#define G f[1]
#define R r
#define GB f[0]
#define RG f[1]

static void yuv2rgb16(Uint8 *lum,   Uint8 *cr,  Uint8 *cb,
                      Uint8 *out, int   rows, int   cols)
{
  int     r_tmp, g_tmp, b_tmp;
  Uint8   f[2];
  Uint8   r   = 0;
  Uint16 *rgb = (Uint16*)f;
  
  Uint16  h, h1, w, w1, w2; 
  Uint8  *dst;
  Uint8  *y, *u, *v;

  y   = lum;
  v   = cr;
  u   = cb;
  dst = out;

  w2 = 0;
  for (h=0; h<cols; h++) 
  {
    h1 = h>>1;
    for (w=0; w<rows; w++)
    {
      w1 = w>>1;
      
      r_tmp = (int)( 1.364*(y[w] - 16) + 2.018*(v[w1] - 128) );
      g_tmp = (int)( 1.364*(y[w] - 16) - 0.813*(u[w1] - 128) - 0.391*(v[w1] - 128) );
      b_tmp = (int)( 1.364*(y[w] - 16) + 1.596*(u[w1] - 128) );
      
      if (r_tmp < 0) 
        r_tmp=0;
      else if (r_tmp > 255)
        r_tmp=255;
      
      if (g_tmp < 0) 
        g_tmp=0;
      else if (g_tmp > 255)
        g_tmp=255;
      
      if (b_tmp < 0) 
        b_tmp=0;
      else if (b_tmp > 255)
        b_tmp=255;
        
      R = (Uint8)r_tmp;
      G = (Uint8)g_tmp;
      B = (Uint8)b_tmp;    
      
      R    = R & 0xF8;      // r:    RRRRR000
      G    = G>>2;          // f[1]: 00GGGGGG
      *rgb = (*rgb>>3);     // f[1]: 00000GGG f[0]: GGGBBBBB
      G    = G | R;         // f[1]: RRRRRGGG
      
      dst[w2]   = GB;  // = f[0]
      dst[w2+1] = RG;  // = f[1]
      
      w2 += 2;
    }
    y += rows;
    u  = cr + rows/2 * h1;
    v  = cb + rows/2 * h1;
  }
} 
			  
			  
			  
			  
			   
